import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { BankRatesService } from '../services/bank-rates.service';
import { getCacheData } from '../utils/helper';
import { AgentWiseRatesService } from '../services/agent-wise-rates.service';
import { AgentWiseCommissionService } from '../services/agent-wise-commission.service';
import { ToastrService } from 'ngx-toastr';
import { AgencyService } from '../services/agency-rates.service';



interface ValidationRate {
  minRate: number | null;
  maxRate: number | null;
}

interface ValidationResult {
  offerRate?: ValidationRate;
  settlementRate?: ValidationRate;
  RevaluationRate?: ValidationRate;
  BidRate?: ValidationRate;
}

@Component({
  selector: 'app-agent-wise-rates',
  templateUrl: './agent-wise-rates.component.html',
  styleUrls: ['./agent-wise-rates.component.css'],
})
export class AgentWiseRatesComponent {
  @ViewChild('manageAgentWiseRates') manageAgenctWiseRatesModal;
  bsModalRef: BsModalRef;

  type: any = 'add';
  validationResults: ValidationResult[] = []; // Updated validationResults array
  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';
  receivingInsertCountries: any = [];
  isLoading: boolean = false;
  agentWiseRates: any = [];

  receivingAgents: any = [];
  selectedReceivingAgent: any = 'All';
  receivingInsertAgents: any = [];

  sendingCountries: any = [];
  selectedSendingCountry: any = 'All';
  sendingInsertCountries: any = [];

  sendingAgents: any = [];
  selectedSendingAgent: any = 'All';
  sendingInsertAgents: any = [];

  sendingCurrencies: any = [];
  receivingCurrencies: any = [];
  modeOfPayments: any = [];
  settlementCurrencies: any = [];

  manageAgencyRatesForm = new FormGroup({
    ID: new  FormControl(''),
    FromCountryID: new FormControl('', Validators.required),
    FromCurrencyID: new FormControl('', Validators.required),
    FromAgentID: new FormControl('', Validators.required),
    ToCountryID: new FormControl('', Validators.required),
    ToCurrencyID: new FormControl('', Validators.required),
    ToAgentID: new FormControl('', Validators.required),
    ModeOfPaymentID: new FormControl('', Validators.required),
    SettlementCurrency: new FormControl('', Validators.required),
    Bid_Rate: new FormControl('', Validators.required),
    Offer_Rate: new FormControl('', Validators.required),
    Revalution_Rate: new FormControl('', Validators.required),
    Settelment_Rate: new FormControl('', Validators.required),
    AutoRateflag: new FormControl(false),
  });

  constructor(
    private bankService: BankRatesService,
    private agentWiseService: AgentWiseRatesService,
    private agentWiseCommissionService: AgentWiseCommissionService,
    private modalService: BsModalService,
    private toastr: ToastrService,
    private agencyService: AgencyService,
  ) { }

  ngOnInit(): void {
    this.isLoading = true;
    this.getSendingCountry();
    this.getSendingAgent();
    this.getReceivingCountry();
    this.getReceivingAgent();
    this.getAllAgentWiseRates();
  }

  getAllAgentWiseRates(
    sendingCountryId: any = 0,
    sendingAgentId: any = 0,
    receivingCountryId: any = 0,
    receivingAgentId: any = 0
  ) {
    this.isLoading = true;
    this.agentWiseService
      .getAllAgentWiseRates(
        getCacheData('admin_token'),
        getCacheData('username'),
        sendingCountryId,
        sendingAgentId,
        receivingCountryId,
        receivingAgentId
      )
      .subscribe({
        next: (res: any) => {
          this.agentWiseRates = res;
        },
        error: (err) => {
          this.agentWiseRates = [];
          this.isLoading = false;

        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }

  getReceivingCountry(userId: any = 0, toCountryId = 0, receiveAgentId = 0, currency_name = '', mode_of_payment = '', settlement_currency = '') {
    this.bankService
      .getReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
          this.receivingInsertCountries = [...res];
          if (this.type == 'edit') {
            this.getReceivingCurrencyInsert(toCountryId, currency_name);
            this.getReceivingAgent(toCountryId, receiveAgentId, mode_of_payment, settlement_currency);
          }
        },
        error: (err) => {
          this.receivingCountries = [];
        },
        complete: () => { },
      });
  }

  getReceivingAgent(countryId: any = 0, receivingAgentId = 0, mode_of_payment = '', settlement_currency = '') {
    this.bankService
      .getReceivingAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingAgents = res;
          this.receivingInsertAgents = [...res];
          if (this.type == 'edit') {
            const receiveAgentId = this.receivingAgents.find((a) => a.Agent_ID == receivingAgentId)?.Agent_ID;
            this.manageAgencyRatesForm.patchValue({
              ToAgentID: receiveAgentId,
            });
            this.getModeOfPayment(countryId, receiveAgentId, mode_of_payment);
            this.getSettlementCurrency(receiveAgentId, settlement_currency);
          }

        },
        error: (err) => {
          this.receivingAgents = [];
        },
        complete: () => { },
      });
  }

  getSendingAgent(type: any = 0, sendingcountryId: any = 0, sendingAgentId = 0) {
    this.agentWiseService
      .getSendingAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        type,
        sendingcountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingAgents = res;
          this.sendingInsertAgents = [...res];
          if (this.type == 'edit') {
            const sendAgentId = this.sendingAgents.find((a) => a.Agent_ID == sendingAgentId)?.Agent_ID;
            this.manageAgencyRatesForm.patchValue({
              FromAgentID: sendAgentId,
            });
          }
        },
        error: (err) => {
          this.sendingAgents = [];
        },
        complete: () => { },
      });
  }

  getSendingCountry(userId: any = 0, fromCountryId = 0, sendingAgentId = 0, currency_name = '') {
    this.agentWiseService
      .getSendingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCountries = res;
          this.sendingInsertCountries = [...res];
          if (this.type == 'edit') {
            this.getSendingAgent(0, fromCountryId, sendingAgentId);
            this.getSendCurrencyInsert(fromCountryId, currency_name);
          }
        },
        error: (err) => {
          this.sendingCountries = [];
        },
        complete: () => { },
      });
  }

  onSendingInsertCountryChange(countryId) {
    if (countryId) {
      this.getSendingAgent(0, countryId);
      this.getSendCurrencyInsert(countryId);
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg', // Set the modal size to large
    });

    // Wait for the modal to be shown
    this.modalService.onShown.subscribe(() => {
      // Find the modal content element
      const modalContent = document.querySelector('.modal-lg .modal-content');
      if (modalContent) {
        // Set the height of the modal content to 40vh (40% of the viewport height)
        // modalContent.setAttribute('style', 'height: 40vh;');
      }
    });

  }

  openAgencyRatesModal(type: any) {
    if (type == 'add') {
      this.manageAgencyRatesForm.reset({
        FromCountryID: 'None',
      });
    }
    this.type = type;
    this.openModal(this.manageAgenctWiseRatesModal);
  }

  get f() {
    return this.manageAgencyRatesForm.value;
  }

  addAgencyWiseRates() {
    if (!this.manageAgencyRatesForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.FromCountryID == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    let payload;
    payload = {
      FromCountryID: this.f.FromCountryID,
      FromCurrencyID: this.f.FromCurrencyID,
      FromAgentID: this.f.FromAgentID,
      ToCountryID: this.f.ToCountryID,
      ToCurrencyID: this.f.ToCurrencyID,
      ToAgentID: this.f.ToAgentID,
      ModeOfPaymentID: this.f.ModeOfPaymentID,
      SettlementCurrency: this.f.SettlementCurrency,
      Bid_Rate: this.f.Bid_Rate,
      Offer_Rate: this.f.Offer_Rate,
      Revalution_Rate: this.f.Revalution_Rate,
      Settelment_Rate: this.f.Settelment_Rate,
      EnteredBy: getCacheData('G_UserID'),
    };

    if (this.type === 'edit') {



      const formValues = this.manageAgencyRatesForm.value;
      const { BidRate } = this.validationResults[1];
  const {offerRate}= this.validationResults[2];
  const {settlementRate}= this.validationResults[3];
  const {RevaluationRate}= this.validationResults[4];
      // Validate Bid Rate
     
      if (BidRate) {
        if (Number(formValues.Bid_Rate) <= BidRate.minRate || Number(formValues.Bid_Rate) >= BidRate.maxRate) {
          alert(`Bid Rate should be between ${BidRate.minRate} and ${BidRate.maxRate}`);
          return;
        }
      }
  
      // Validate Offer Rate
      if (offerRate) {
        if (Number(formValues.Offer_Rate) <= offerRate.minRate || Number(formValues.Offer_Rate) >= offerRate.maxRate) {
          alert(`Offer Rate should be between ${offerRate.minRate} and ${offerRate.maxRate}`);
          return;
        }
      }
  
      // Validate Revaluation Rate
      if (RevaluationRate) {
        if (Number(formValues.Revalution_Rate) <= RevaluationRate.minRate || Number(formValues.Revalution_Rate) >= RevaluationRate.maxRate) {
          alert(`Revaluation Rate should be between ${RevaluationRate.minRate} and ${RevaluationRate.maxRate}`);
          return;
        }
      }
  
      // Validate Settlement Rate
      if (settlementRate) {
        if (Number(formValues.Settelment_Rate) <= settlementRate.minRate || Number(formValues.Settelment_Rate) >= settlementRate.maxRate) {
          alert(`Settlement Rate should be between ${settlementRate.minRate} and ${settlementRate.maxRate}`);
          return;
        }
      }

      payload = [
        {
          FromCountryID: this.f.FromCountryID,
          FromCurrencyID: this.f.FromCurrencyID,
          FromAgentID: this.f.FromAgentID,
          ToCountryID: this.f.ToCountryID,
          ToCurrencyID: this.f.ToCurrencyID,
          ToAgentID: this.f.ToAgentID,
          ModeOfPaymentID: this.f.ModeOfPaymentID,
          SettlementCurrency: this.f.SettlementCurrency,
          Bid_Rate: this.f.Bid_Rate,
          Offer_Rate: this.f.Offer_Rate,
          Revalution_Rate: this.f.Revalution_Rate,
          Settelment_Rate: this.f.Settelment_Rate,
          EnteredBy:getCacheData('G_UserID'),
          Active: this.f.AutoRateflag ? this.f.AutoRateflag : false,

        },
      ];
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addAgentWiseExchangeRates'
        : 'updateAgentWiseExchangeRates';

    this.agentWiseService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        const message = this.type === 'edit' ? 'updated' : 'added';
        alert(res);
      },
      error: (err) => {
        this.modalService.hide();
        // alert(err?.error?.message || 'Something went wrong');
        this.toastr.error(err?.error?.message )

      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  onReceivingCountryChange() {
    if (this.selectedReceivingCountry != 'All') {
      this.getReceivingAgent(this.selectedReceivingCountry);
    }
  }

  closeModal() {
    this.bsModalRef.hide();
  }

  editAgentWiseRates(rates: any) {
    this.type = 'edit';
    const sendCountryId = this.sendingInsertCountries.find((c) => c.FromCountry_ID == rates.SendCountryID)?.FromCountry_ID;
    const receiveCountryId = this.receivingCountries.find((c) => c.Country_ID == rates.ReceCountryID)?.Country_ID;

    this.getSendingCountry(0, sendCountryId, rates.Agent_ID, rates.SendingCurrency);
    this.getReceivingCountry(0, receiveCountryId, rates.ToAgent_ID, rates.ReceiveCurrency, rates.ModeOfPayment, rates.SettelmentCurrency);

console.log(rates,"{")

this.validateBidRate(1,rates?.RatePer, rates?.BidRate)
this.validateOfferRate(2,rates?.RatePer, rates?.OfferRate)
this.validateSettlementRate(3,rates?.RatePer, rates?.SettelmentRate)
this.validateRevaluationRate(4,rates?.RatePer, rates?.RevalutionRate)

    this.manageAgencyRatesForm.patchValue({
      ID:rates.ID,
      FromCountryID: sendCountryId,
      ToCountryID: receiveCountryId,
      Bid_Rate: rates?.BidRate,
      Offer_Rate: rates?.OfferRate,
      Revalution_Rate: rates?.RevalutionRate,
      Settelment_Rate: rates?.SettelmentRate,
      AutoRateflag: rates?.Active,
    });
  }


  searchBankRates() {
    let sendingCountryId: number | null = 0;
    let sendingAgentId: number | null = 0;
    let receivingCountryId: number | null = 0;
    let receivingAgentId: number | null = 0;
    if (this.selectedSendingCountry !== 'All') {
      sendingCountryId = this.selectedSendingCountry;
    }

    if (this.selectedSendingAgent !== 'All') {
      sendingAgentId = this.selectedSendingAgent;
    }

    if (this.selectedReceivingCountry !== 'All') {
      receivingCountryId = this.selectedReceivingCountry;
    }
    if (this.selectedReceivingAgent !== 'All') {
      receivingAgentId = this.selectedReceivingAgent;
    }
    this.getAllAgentWiseRates(
      sendingCountryId,
      sendingAgentId,
      receivingCountryId,
      receivingAgentId
    );
  }

  onSendingCountryChange() {
    if (this.selectedSendingCountry != 'All') {
      this.getSendingAgent(0, this.selectedSendingCountry);
    }
  }

  onReceiveCountryInsertChange(countryId) {
    if (countryId) {
      this.getReceivingCurrencyInsert(countryId);
    }
  }

  onReceiveCurrencyInsertChange(countryId) {
    if (this.f.ToCountryID) {
      this.getReceivingAgent(this.f.ToCountryID);
    }
  }


  getSendCurrencyInsert(countryId: any = 0, currency_name = '') {
    this.agentWiseCommissionService
      .getSendCurrencyInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCurrencies = res;
          if (this.type == 'edit') {
            const sendCurrencyId = this.sendingCurrencies.find((c) => c.CurrencyShortName == currency_name)?.Currency_Id;
            this.manageAgencyRatesForm.patchValue({
              FromCurrencyID: sendCurrencyId,
            });
          }
        },
        error: (err) => {
          this.sendingCurrencies = [];
        },
        complete: () => { },
      });
  }

  getReceivingCurrencyInsert(countryId: any = 0, currency_name = '') {
    this.bankService
      .getReceivingCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCurrencies = res;
          if (this.type == 'edit') {
            const receiveCurrencyId = this.receivingCurrencies.find((c) => c.CurrencyShortName == currency_name)?.Currency_Id;
            this.manageAgencyRatesForm.patchValue({
              ToCurrencyID: receiveCurrencyId,
            });
          }
        },
        error: (err) => {
          this.receivingCurrencies = [];
        },
        complete: () => { },
      });
  }

  onReceiveAgentInsertChange(agentId) {
    if (this.f.ToCountryID && agentId) {
      this.getModeOfPayment(this.f.ToCountryID, agentId);
      this.getSettlementCurrency(agentId);
    }
  }

  getModeOfPayment(targetCountryId: any = 0, receiveAgentId: any = 0, mode_of_payment = '') {
    this.agentWiseService
      .getModeOfPayment(targetCountryId, receiveAgentId)
      .subscribe({
        next: (res: any) => {
          this.modeOfPayments = res;
          if (this.type == 'edit') {
            const modeofPaymentId = this.modeOfPayments.find((m) => m.ModeOfPayment_Name == mode_of_payment)?.ModeOfPayment_ID;
            this.manageAgencyRatesForm.patchValue({
              ModeOfPaymentID: modeofPaymentId
            });
          }
        },
        error: (err) => {
          this.modeOfPayments = [];
        },
        complete: () => { },
      });
  }

  getSettlementCurrency(agentId: any = 0, settlement_currency = '') {
    this.agentWiseService.getSettlementCurrency(agentId).subscribe({
      next: (res: any) => {
        this.settlementCurrencies = res;
        if (this.type == 'edit') {
          const settlementCurrencyId = this.settlementCurrencies.find((s) => s.CurrencyShortName == settlement_currency)?.Currency_Id;
          this.manageAgencyRatesForm.patchValue({
            SettlementCurrency: settlementCurrencyId
          });
          this.openAgencyRatesModal('edit');
        }
      },
      error: (err) => {
        this.settlementCurrencies = [];
      },
      complete: () => { },
    });
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  updateAgencyWiseRates() {
    if (!this.manageAgencyRatesForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.FromCountryID == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    const formValues = this.manageAgencyRatesForm.value;
    const { BidRate } = this.validationResults[1];
const {offerRate}= this.validationResults[2];
const {settlementRate}= this.validationResults[3];
const {RevaluationRate}= this.validationResults[4];
    // Validate Bid Rate
   
    if (BidRate) {
      if (Number(formValues.Bid_Rate) <= BidRate.minRate || Number(formValues.Bid_Rate) >= BidRate.maxRate) {
        alert(`Bid Rate should be between ${BidRate.minRate} and ${BidRate.maxRate}`);
        return;
      }
    }

    // Validate Offer Rate
    if (offerRate) {
      if (Number(formValues.Offer_Rate) <= offerRate.minRate || Number(formValues.Offer_Rate) >= offerRate.maxRate) {
        alert(`Offer Rate should be between ${offerRate.minRate} and ${offerRate.maxRate}`);
        return;
      }
    }

    // Validate Revaluation Rate
    if (RevaluationRate) {
      if (Number(formValues.Revalution_Rate) <= RevaluationRate.minRate || Number(formValues.Revalution_Rate) >= RevaluationRate.maxRate) {
        alert(`Revaluation Rate should be between ${RevaluationRate.minRate} and ${RevaluationRate.maxRate}`);
        return;
      }
    }

    // Validate Settlement Rate
    if (settlementRate) {
      if (Number(formValues.Settelment_Rate) <= settlementRate.minRate || Number(formValues.Settelment_Rate) >= settlementRate.maxRate) {
        alert(`Settlement Rate should be between ${settlementRate.minRate} and ${settlementRate.maxRate}`);
        return;
      }
    }

    const payload = {
      ID:this.f.ID,
      FromCountryID: this.f.FromCountryID,
      FromCurrencyID: this.f.FromCurrencyID,
      FromAgentID: this.f.FromAgentID,
      ToCountryID: this.f.ToCountryID,
      ToCurrencyID: this.f.ToCurrencyID,
      ToAgentID: this.f.ToAgentID,
      ModeOfPaymentID: this.f.ModeOfPaymentID,
      SettlementCurrency: this.f.SettlementCurrency,
      Bid_Rate: this.f.Bid_Rate,
      Offer_Rate: this.f.Offer_Rate,
      Revalution_Rate: this.f.Revalution_Rate,
      Settelment_Rate: this.f.Settelment_Rate,
      EnteredBy: getCacheData('G_UserID'),
      Active: this.f.AutoRateflag ? this.f.AutoRateflag : false,
    };

    this.agentWiseService.updateAgentWiseExchangeRates(

      payload
    ).subscribe({
      next: (res: any) => {
        alert(res);
        this.modalService.hide(); // Close the modal after successful update
        this.getAllAgentWiseRates();
      },
      error: (err) => {
        // alert(err?.error?.message || 'Something went wrong');
        this.toastr.error(err?.error?.message);
      }
    });
  }


  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  validateOfferRate(index: number, ratePer: any, offerRate: any): void {
    console.log(ratePer,offerRate)
    if (!ratePer || offerRate == null || ratePer === undefined || ratePer === '') {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        offerRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              offerRate: { minRate: parseFloat(MinimumRate), maxRate: parseFloat(MaximumRate) }
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              offerRate: { minRate: null, maxRate: null }
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            offerRate: { minRate: null, maxRate: null }
          };
        },
        complete: () => {},
      });
  }

  validateSettlementRate(index: number, ratePer: any, settlementRate: any): void {


    console.log(ratePer,settlementRate)
    if (!ratePer || settlementRate == null || ratePer === undefined || ratePer === '') {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        settlementRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              settlementRate: { minRate: parseFloat(MinimumRate), maxRate: parseFloat(MaximumRate) }
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              settlementRate: { minRate: null, maxRate: null }
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            settlementRate: { minRate: null, maxRate: null }
          };
        },
        complete: () => {},
      });
  }


  validateRevaluationRate(index: number, ratePer: any, RevaluationRate: any): void {
    if (!ratePer || RevaluationRate == null || ratePer === undefined || ratePer === '') {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        RevaluationRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              RevaluationRate: { minRate: parseFloat(MinimumRate), maxRate: parseFloat(MaximumRate) }
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              RevaluationRate: { minRate: null, maxRate: null }
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            settlementRate: { minRate: null, maxRate: null }
          };
        },
        complete: () => {},
      });
  }

  validateBidRate(index: number, ratePer: any, BidRate: any): void {
    if (!ratePer || BidRate == null || ratePer === undefined || ratePer === '') {
      this.validationResults = [];

      return;
    }

    this.agencyService
      .getvalidation(
        getCacheData('admin_token'),
        getCacheData('username'),
        ratePer,
        BidRate
      )
      .subscribe({
        next: (res: any) => {
          console.log('Service response:', res); // Debugging line
          if (res && res.length > 0) {
            const { MinimumRate, MaximumRate } = res[0];
            this.validationResults[index] = {
              ...this.validationResults[index],
              BidRate: { minRate: parseFloat(MinimumRate), maxRate: parseFloat(MaximumRate) }
            };
            console.log('Validation results:', this.validationResults[index]); // Debugging line
          } else {
            this.validationResults[index] = {
              ...this.validationResults[index],
              BidRate: { minRate: null, maxRate: null }
            };
          }
        },
        error: (err) => {
          // Handle error
          this.validationResults[index] = {
            ...this.validationResults[index],
            settlementRate: { minRate: null, maxRate: null }
          };
        },
        complete: () => {},
      });
  }



}