import { Component, ElementRef, TemplateRef, ViewChild } from '@angular/core';
import { CancellationCheckerService } from '../cancellation-checker.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { AuthService } from '../services/auth.service';
import * as XLSX from 'xlsx';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-cancellation-checker',
  templateUrl: './cancellation-checker.component.html',
  styleUrls: ['./cancellation-checker.component.css'],
})
export class CancellationCheckerComponent {
  bsModalRef: BsModalRef;
  @ViewChild('cancellationCheckerTable') cancellationCheckerTable: ElementRef;

  @ViewChild('viewLog') viewLogModal;
  @ViewChild('view') viewModal;
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;

  viewData: any;
  LogData: any;
  allData: any;
  selectedOption: string = 'transactionNo';
  startDate: any;
  endDate: any;
  transactionNumber: any;
  selectedTransactionType: any = '0';
  selectedFileFormat = '1';
  receiptData: any = {};

  constructor(
    private modalService: BsModalService,
    private CancellationCheckerSvc: CancellationCheckerService,
    private authSvc: AuthService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit() {
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  search() {
    if (this.selectedOption === 'transactionNo' && !this.transactionNumber) {
      alert('Please Enter Transaction No.');
      return;
    }

    if (
      (this.selectedOption === 'byDate' && !this.startDate) ||
      !this.endDate
    ) {
      alert('Please Select Start and End Dates.');
      return;
    }
    if (this.selectedOption === 'transactionNo') {
      const payload = {
        SearchType: 1,
        FromDate: '',
        Todate: '',
        TransactionType: Number(this.selectedTransactionType),
        Search: this.transactionNumber,
      };

      this.CancellationCheckerSvc.getAllRecord(payload).subscribe(
        (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.allData = res;
        }
      );
    } else if (this.selectedOption === 'byDate') {
      const payload = {
        SearchType: 2,
        TransactionType: Number(this.selectedTransactionType),
        FromDate: this.startDate,
        Todate: this.endDate,
        Search: 'None',
      };

      this.CancellationCheckerSvc.getAllRecord(payload).subscribe(
        (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.allData = res;
        }
      );
    }
  }

  getLog(TransactionNumber: any) {
    this.CancellationCheckerSvc.getLog(TransactionNumber).subscribe((res) => {
      this.openModal(this.viewLogModal);

      this.LogData = res;
    });
  }

  viewDetails(TransactionNumber: any) {
    this.CancellationCheckerSvc.View(TransactionNumber).subscribe((res) => {
      this.openModal(this.viewModal);
      this.viewData = res?.[0];
    });
  }

  approve() {
    const payload = {
      G_UserID: getCacheData('G_UserID'),
      TransactionNumber: String(this.viewData?.TransactionNumber),
      CanCode: this.viewData?.CanCode,
      ddlRevReason: 'None',
      App_ID: String(this.viewData?.App_ID),
      txtR: this.viewData?.Remarks ? this.viewData?.Remarks : '',
      LogID: String(this.viewData?.LogID),
      Old_Status: String(this.viewData?.Old_Status),
    };

    this.CancellationCheckerSvc.approve(payload).subscribe({
      next: (res: any) => {
        if (res) {
          this.search();
          this.modalService.hide();
          alert(res?.[0]?.Message);
          return;
        }
      },
      error: (err) => {
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  reject() {
    const payload = {
      G_UserID: getCacheData('G_UserID'),
      TransactionNumber: String(this.viewData?.TransactionNumber),
      CanCode: this.viewData?.CanCode,
      ddlRevReason: 'None',
      App_ID: String(this.viewData?.App_ID),
      txtR: this.viewData?.Remarks ? this.viewData?.Remarks : '',
      LogID: String(this.viewData?.LogID),
      Old_Status: String(this.viewData?.Old_Status),
    };
    this.CancellationCheckerSvc.reject(payload).subscribe({
      next: (res: any) => {
        if (res) {
          this.search();
          this.modalService.hide();
          alert(res?.[0]?.Message);
          return;
        }
      },
      error: (err) => {
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  recheck() {
    const payload = {
      G_UserID: getCacheData('G_UserID'),
      TransactionNumber: String(this.viewData?.TransactionNumber),
      OrgMsgID: String(this.viewData?.OrgMsgID),
      CanCode: this.viewData?.CanCode,
      txtR: this.viewData?.Remarks ? this.viewData?.Remarks : '',
      LogID: String(this.viewData?.LogID),
      Old_Status: this.viewData?.Old_Status,
    };
    this.CancellationCheckerSvc.Recheck(payload).subscribe({
      next: (res: any) => {
        if (res) {
          this.search();
          this.modalService.hide();
          alert(res?.[0]?.Message);
          return;
        }
      },
      error: (err) => {
        // alert(err?.error?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.cancellationCheckerTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Cancellation_Checker${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.TransactionNumber;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }
}
