import { Component } from '@angular/core';
import { CancellationRequestService } from '../services/cancellation-request.service';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-cancellation-request',
  templateUrl: './cancellation-request.component.html',
  styleUrls: ['./cancellation-request.component.css'],
})
export class CancellationRequestComponent {
  statusList: any = [];
  selectedStatusList: any;
  transactionNo: any = '';
  remark: any = '';

  constructor(
    private cancelRequestSvc: CancellationRequestService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit(): void {
    this.getStatusList();
  }

  getStatusList() {
    this.cancelRequestSvc.getStatusList().subscribe({
      next: (res: any) => {
        this.statusList = res;
        this.selectedStatusList = res?.[0]?.TrStatusID;
      },
      error: () => {
        this.statusList = [];
      },
      complete: () => {},
    });
  }

  updateCancelRequest() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction Number!');
      return;
    }

    if (!this.selectedStatusList) {
      alert('Please Select Status!');
      return;
    }

    if (!this.remark) {
      alert('Please Enter Remarks!');
      return;
    }
    let payload: any = {};
    payload = {
      SRTRNo: this.transactionNo,
      txtRemarks: this.remark,
      UserID: Number(getCacheData('G_UserID')),
      UserGroup: Number(getCacheData('G_UserGroup')),
      Agent_Branch_ID: 0,
      SubUserGroup_Id: Number(getCacheData('SubUserGroup_Id')),
      TrStatusID: this.selectedStatusList,
    };
    this.cancelRequestSvc.updateCancelRequest(payload).subscribe({
      next: () => {
        this.clearFields();
      },
      error: (err) => {
        // alert(err?.error?.message || 'Something went wrong');
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {
        this.clearFields();
      },
    });
  }

  clearFields() {
    this.transactionNo = '';
    this.remark = '';
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
