import { Component, OnInit } from '@angular/core';
import { CommissionIncomeService } from '../services/commission-income.service';

@Component({
  selector: 'app-commission-income',
  templateUrl: './commission-income.component.html',
  styleUrls: ['./commission-income.component.css'],
})
export class CommissionIncomeComponent implements OnInit {
  reportTypes: any[] = [];
  countries: any[] = [];
  currencies: any[] = [];
  agents: any[] = [];
  receivingCountries: any[] = [];
  reportSelectedValue: string;
  SendAgent: any[] = [];
  unpaidPrincipalData: any[] = [];
  fromDate: string | null = null;
  toDate: string | null = null;
  selectedCurrencyId: any;
  AgentID: any; // Variable to store the selected currency ID
  CountryID: any; // Variable to store the selected currency ID
  selectedSendingAgentId: any;
  selectedReceivingCountryId: any;
  agencyCommissionIncomeData: any[] = [];
  globalTransactionsIncomeSummaryReportData: any[] = [];
  agencyExchangeEarningsData: any[] = []; // Variable to store the data
  constructor(private commissionService: CommissionIncomeService) {}

  ngOnInit(): void {
    this.loadReportTypes();
    this.loadCountries();
    this.loadCurrencies();
    this.loadReceivingCountries();
    this.loadSendAgent();
  }

  loadReportTypes() {
    this.commissionService.getReportType().subscribe(
      (data: any[]) => {
        this.reportTypes = data;
      },
      (error) => {
        console.error('Error fetching report types:', error);
      }
    );
  }

  loadCountries() {
    this.commissionService.getReportCountry().subscribe(
      (data: any[]) => {
        this.countries = data;
      },
      (error) => {
        console.error('Error fetching countries:', error);
      }
    );
  }

  loadSendAgent() {
    this.commissionService.ReportSendAgent().subscribe(
      (data: any[]) => {
        this.SendAgent = data;
      },
      (error) => {
        console.error('Error fetching countries:', error);
      }
    );
  }

  loadCurrencies() {
    this.commissionService.getReportCurrency().subscribe(
      (data: any[]) => {
        this.currencies = data;
      },
      (error) => {
        console.error('Error fetching currencies:', error);
      }
    );
  }

  loadAgents(countryId: any) {
    this.commissionService.getReportAgent(countryId).subscribe(
      (data: any[]) => {
        this.agents = data;
      },
      (error) => {
        console.error('Error fetching agents:', error);
      }
    );
  }

  loadReceivingCountries() {
    this.commissionService.ReportReceivingCountry().subscribe(
      (data: any[]) => {
        this.receivingCountries = data;
      },
      (error) => {
        console.error('Error fetching receiving countries:', error);
      }
    );
  }

  onReportTypeChange(reportType: string) {
    this.reportSelectedValue = reportType;
    console.log(reportType);
  }

  onCountryChange(countryId: any) {
    this.loadAgents(countryId);
    this.CountryID = countryId;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  viewReport() {
    if (!this.reportSelectedValue) {
      alert('Please Select Report.');
      return;
    }
    if (this.reportSelectedValue == 'Unpaid Principal Report') {
      if (!this.fromDate) {
        alert('Please Select From Date.');
        return;
      }
      if (!this.toDate) {
        alert('Please Select To Date.');
        return;
      }
      if (!this.selectedCurrencyId) {
        alert('Please Select Currency.');
        return;
      }
      this.loadUnpaidPrincipalGrid(
        this.fromDate,
        this.toDate,
        this.selectedCurrencyId
      );
    } else if (this.reportSelectedValue == 'Unpaid Commission Report') {
      if (!this.fromDate) {
        alert('Please Select From Date.');
        return;
      }
      if (!this.toDate) {
        alert('Please Select To Date.');
        return;
      }
      if (!this.selectedCurrencyId) {
        alert('Please Select Currency.');
        return;
      }
      this.loadUnpaidCommissionGrid(
        this.fromDate,
        this.toDate,
        this.selectedCurrencyId
      );
    } else if (this.reportSelectedValue == 'Agency Commission Income Report') {
      if (!this.fromDate) {
        alert('Please Select From Date.');
        return;
      }
      if (!this.toDate) {
        alert('Please Select To Date.');
        return;
      }
      if (!this.selectedCurrencyId) {
        alert('Please Select Currency.');
        return;
      }
      this.loadAgencyCommissionIncomeGrid(
        this.fromDate,
        this.toDate,
        this.selectedCurrencyId
      );
    } else if (this.reportSelectedValue == 'Agency Exchange Earnings Report') {
      if (!this.fromDate) {
        alert('Please Select From Date.');
        return;
      }
      if (!this.toDate) {
        alert('Please Select To Date.');
        return;
      }
      if (!this.selectedSendingAgentId) {
        alert('Please Select Sending Agent.');
        return;
      }
      this.loadAgencyExchangeEarningsGrid(
        this.fromDate,
        this.toDate,
        this.selectedSendingAgentId
      );
    } else if (this.reportSelectedValue == 'Global Trxns Income Summary') {
      if (!this.fromDate) {
        alert('Please Select From Date.');
        return;
      }
      if (!this.toDate) {
        alert('Please Select To Date.');
        return;
      }
      if (!this.selectedSendingAgentId) {
        alert('Please Select Sending Agent.');
        return;
      }
      if (!this.CountryID) {
        alert('Please Select Country.');
        return;
      }
      this.loadGlobalTransactionsIncomeSummaryReportGrid(
        this.fromDate,
        this.toDate,
        this.selectedSendingAgentId,
        this.CountryID
      );
    } else if (
      this.reportSelectedValue == 'Global Trxns Income Summary Corridorwise'
    ) {
      if (!this.fromDate) {
        alert('Please Select From Date.');
        return;
      }
      if (!this.toDate) {
        alert('Please Select To Date.');
        return;
      }
      if (!this.selectedSendingAgentId) {
        alert('Please Select Sending Agent.');
        return;
      }
      if (!this.CountryID) {
        alert('Please Select Country.');
        return;
      }
      if (!this.selectedReceivingCountryId) {
        alert('Please Select Receiving Country.');
        return;
      }
      this.loadGlobalTransactionsIncomeSummaryCorridorwiseGrid(
        this.fromDate,
        this.toDate,
        this.selectedSendingAgentId,
        this.CountryID,
        this.selectedReceivingCountryId
      );
    } else {
      // Handle default case or do nothing
    }
  }
  onSendingAgentChange(agentId: any) {
    this.selectedSendingAgentId = agentId;
  }
  loadUnpaidPrincipalGrid(FromDate: any, ToDate: any, CurrencyID: any) {
    console.log('called');
    this.commissionService
      .ReportUnpaidPrincipalGrid(FromDate, ToDate, CurrencyID)
      .subscribe(
        (data: any[]) => {
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          this.unpaidPrincipalData = data; // Save the response data to the variable
        },
        (error) => {
          console.error('Error fetching unpaid principal grid:', error);
        }
      );
  }
  loadUnpaidCommissionGrid(FromDate: any, ToDate: any, CurrencyID: any) {
    this.commissionService
      .ReportUnpaidCommissionGrid(FromDate, ToDate, CurrencyID)
      .subscribe(
        (data: any[]) => {
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          // Handle the response data as needed
        },
        (error) => {
          console.error('Error fetching unpaid principal grid:', error);
        }
      );
  }
  loadAgencyCommissionIncomeGrid(FromDate: any, ToDate: any, CurrencyID: any) {
    this.commissionService
      .ReportAgencyCommissionIncomeGrid(FromDate, ToDate, CurrencyID)
      .subscribe(
        (data: any[]) => {
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          this.agencyCommissionIncomeData = data;
        },
        (error) => {
          console.error('Error fetching unpaid principal grid:', error);
        }
      );
  }
  loadAgencyExchangeEarningsGrid(FromDate: any, ToDate: any, AgentID: any) {
    this.commissionService
      .ReportAgencyExchangeEarningsGrid(FromDate, ToDate, AgentID)
      .subscribe(
        (data: any[]) => {
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          this.agencyExchangeEarningsData = data; // Assign the response data to the variable
        },
        (error) => {
          console.error('Error fetching unpaid principal grid:', error);
        }
      );
  }
  loadGlobalTransactionsIncomeSummaryReportGrid(
    FromDate: any,
    ToDate: any,
    AgentID: any,
    CountryID: any
  ) {
    this.commissionService
      .ReportGlobalTransactionsIncomeSummaryReportGrid(
        FromDate,
        ToDate,
        AgentID,
        CountryID
      )
      .subscribe(
        (data: any[]) => {
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          this.globalTransactionsIncomeSummaryReportData = data;
        },
        (error) => {
          console.error('Error fetching unpaid principal grid:', error);
        }
      );
  }

  loadGlobalTransactionsIncomeSummaryCorridorwiseGrid(
    FromDate: any,
    ToDate: any,
    AgentID: any,
    CountryID: any,
    ReceCountryID: any
  ) {
    this.commissionService
      .ReportGlobalTransactionsIncomeSummaryCorridorwiseGrid(
        FromDate,
        ToDate,
        AgentID,
        CountryID,
        ReceCountryID
      )
      .subscribe(
        (data: any[]) => {
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          this.globalTransactionsIncomeSummaryReportData = data;
        },
        (error) => {
          console.error('Error fetching unpaid principal grid:', error);
        }
      );
  }

  onFromDateChange(event: any) {
    this.fromDate = event.target.value;
  }

  onToDateChange(event: any) {
    this.toDate = event.target.value;
  }
  onCurrencyChange(selectedCurrencyId: any) {
    this.selectedCurrencyId = selectedCurrencyId;
  }

  onReceivingCountryChange(countryId: any) {
    this.selectedReceivingCountryId = countryId;
  }
}
