import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { ManageCountryService } from '../manage-country.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-country-management',
  templateUrl: './country-management.component.html',
  styleUrls: ['./country-management.component.css'],
})
export class CountryManagementComponent implements OnInit {
  @ViewChild('manageCountry') manageCountryModal;
  @ViewChild('manageCountryView') manageCountryViewModal;
  @ViewChild('viewCountry') viewCountryModal; 
  bsModalRef: BsModalRef;
  countries: any = [];
  allCountries: any = [];
  selectedCountry: any = 'All';
  type: any = 'add';
  currencies: any = [];
  localCurrencies: any = [];
  internationalTransactionTypes: any = [];
  domesticTransactionTypes: any = [];
  isLoading: boolean = false;

  listofCurrencies: any = [];
  trAmountLimits: any = [];
  trAmountLimitBank: any = [];
  trAmountLimitDraft: any = [];
  trAmountLimitWalt: any = [];
  outboundIDReqirementThreshold: any = [];

  textareaHeight: number = 76;
  approveReason: any = '';

  manageCountryForm = new FormGroup({
    CountryID: new FormControl(),
    BaseCurrencyID: new FormControl('None', Validators.required),
    CountryName: new FormControl('', [
      Validators.required,
      Validators.pattern(/^[A-Za-z ]+$/)  // Allows only letters and spaces
    ]),
    CountryShortName:new FormControl('', [
      Validators.required,
      Validators.pattern(/^[A-Za-z ]+$/)  // Allows only letters and spaces
    ]),
    AllowedInternationalTransactionType: new FormControl(
      'None',
      Validators.required
    ),
    AllowedDomesticTransactionType: new FormControl(
      'None',
      Validators.required
    ),
    TotalInboundThresholdPerYearPerPerson: new FormControl(
      '',
      Validators.required
    ),
    MaximumTransactionsPerYearPerPersonToOneBeneficiary: new FormControl(
      '',
      Validators.required
    ),
    TransactionYearStartDate: new FormControl('', Validators.required),
    TransactionYearEndDate: new FormControl('', Validators.required),
    OutboundIDRequirementThreshold: new FormControl(''),
    // OutboundThresholdPerTransaction: new FormControl('', Validators.required),
    OutboundThresholdPerMonthTransaction: new FormControl(
      '',
      Validators.required
    ),
    OutboundThresholdPerDayTransaction: new FormControl(
      '',
      Validators.required
    ),
    IsAsian: new FormControl(false, Validators.required),
    // Active: new FormControl(false, Validators.required),
    // Currencies: new FormControl([], Validators.required),
    CurrenciesTrAmtLimit: new FormControl(''),
    CurrenciesTrAmtLimit_Bank: new FormControl(''),
    CurrenciesTrAmtLimit_DD: new FormControl(''),
    CurrenciesTrAmtLimit_WALT: new FormControl(''),
    // CurrenciesOutboundIDReqirementThreshold: new FormControl(
    //   [],
    //   Validators.required
    // ),
    CurrencyName: new FormControl(''),
    InternationalTransactionType: new FormControl(''),
    DomesticTransactionType: new FormControl(''),
    OutboundIdRequirement: new FormControl(''),
    OutboundThresholdPerTransaction: new FormControl(''),
    Approved: new FormControl(''),
    Asian: new FormControl(''),
    UpdatedBaseCurrencyId: new FormControl(''),
  });

  constructor(
    private countryService: ManageCountryService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) { }

  ngOnInit(): void {
    this.getAllCountries();
    this.getCurrency();
    this.getAllowedInternationalTransaction();
    this.getAllowedDomesticTransaction();
  }

  getAllCountries(country_id = 0) {
    this.isLoading = true;
    this.countryService
      .getCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        country_id
      )
      .subscribe({
        next: (res: any) => {
          this.countries = res;
          this.allCountries = [...res];
        },
        error: (err) => {
          this.countries = [];
          this.isLoading = false;

        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }

  searchCountry() {
    if (this.selectedCountry != 'All') {
      this.getAllCountries(this.selectedCountry);
    } else {
      this.getAllCountries();
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg', // 'modal-sm' for small, 'modal-lg' for large
    });
  }

  openCountryModal(type: any) {
    this.type = type;
console.log(type,"type")
    if (type === 'add') {
      this.manageCountryForm.reset({
        BaseCurrencyID: 'None',
        AllowedDomesticTransactionType: 'None',
        AllowedInternationalTransactionType: 'None',
        CountryName: '',
        CountryShortName: '',
        CurrenciesTrAmtLimit: '',
        CurrenciesTrAmtLimit_Bank: '',
        CurrenciesTrAmtLimit_DD: '',
        CurrenciesTrAmtLimit_WALT: '',
        MaximumTransactionsPerYearPerPersonToOneBeneficiary: '',
        OutboundIDRequirementThreshold: '',
        OutboundThresholdPerDayTransaction: '',
        OutboundThresholdPerMonthTransaction: '',
        TotalInboundThresholdPerYearPerPerson: '',
        TransactionYearEndDate: '',
        TransactionYearStartDate: '',
      });
      this.listofCurrencies = [];
      this.trAmountLimits = [];
      this.trAmountLimitBank = [];
      this.trAmountLimitDraft = [];
      this.trAmountLimitWalt = [];
      this.outboundIDReqirementThreshold = [];
      this.openModal(this.manageCountryModal);
    }else if (type === 'view') {
      this.openModal(this.manageCountryViewModal);
    }else if (type === "edit"){
      this.openModal(this.manageCountryModal);
    }

   
  }

  getCurrency() {
    this.countryService
      .getCurrency(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.currencies = res;
          this.getLocalCurrency();
        },
        error: (err) => {
          this.currencies = [];
        },
        complete: () => { },
      });
  }

  getLocalCurrency() {
    if (this.currencies.length) {
      this.localCurrencies = this.currencies;
    }
  }

  getAllowedInternationalTransaction() {
    this.countryService
      .getAllowedInternationalTransaction(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.internationalTransactionTypes = res;
        },
        error: (err) => {
          this.internationalTransactionTypes = [];
        },
        complete: () => { },
      });
  }

  getAllowedDomesticTransaction() {
    this.countryService
      .getAllowedDomesticTransaction(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.domesticTransactionTypes = res;
        },
        error: (err) => {
          this.domesticTransactionTypes = [];
        },
        complete: () => { },
      });
  }

  get f() {
    return this.manageCountryForm.value;
  }

  // Helper method to get currency short name
  getCurrencyShortName(currencyId: any): any | undefined {
    return this.currencies.find((c) => c.Currency_Id === currencyId)?.CurrencyShortName;
  }

  // Helper method to check if the currency name exists
  checkCurrencyExistence(currencyName: string): boolean {
    return this.listofCurrencies.some((c) => c.Value === currencyName);
  }

  // Reusable function to add a new item to the list
  addNewItem(list, valueKey, newValue, idKey) {
    if (newValue != null && newValue !== '') {
      const numericValue = newValue;

      let newCurrency = {
        [idKey]: list.length > 0 ? list[list.length - 1][idKey] + 1 : 1,
        [valueKey]: numericValue,
      };
      list.push(newCurrency);
      // // Check if the value already exists in the list
      // const isValueExists = list.some(
      //   (currency) => currency[valueKey] === numericValue
      // );

      // // If the value doesn't exist, add a new item
      // if (!isValueExists) {
      // }
    }
  }

  // Refactored addOutBoundThersholdTransaction function
  addOutBoundThersholdTransaction() {
    if (this.f.BaseCurrencyID == 'None') {
      alert('Please Select Currency');
      return;
    }
    if (this.f.CurrenciesTrAmtLimit == '') {
      alert('Please Fill Cash');
      return;
    }
    if (this.f.CurrenciesTrAmtLimit_Bank == '') {
      alert('Please Fill Bank');
      return;
    }

    if (this.f.CurrenciesTrAmtLimit_DD == '') {
      alert('Please Fill Draft');
      return;
    }
    if (this.f.CurrenciesTrAmtLimit_WALT == '') {
      alert('Please Fill Walt');
      return;
    }
    if (this.f.OutboundIDRequirementThreshold == '') {
      alert('Please Fill Outbound ID Reqirement Threshold (In Amount)');
      return;
    }

    const currencyId = this.manageCountryForm.get('BaseCurrencyID')?.value;

    const currencyName = this.getCurrencyShortName(currencyId);

    if (this.checkCurrencyExistence(currencyName)) {
      alert('This currency already exists in the list');
      return;
    }

    if (this.f.BaseCurrencyID != 'None') {
      this.addNewItem(
        this.listofCurrencies,
        'Value',
        // this.f.BaseCurrencyID,
        currencyName,
        'Id'
      );
    }

    if (this.f.CurrenciesTrAmtLimit?.length) {
      this.addNewItem(
        this.trAmountLimits,
        'Value',
        this.f.CurrenciesTrAmtLimit,
        'Id'
      );
    }
    if (this.f.CurrenciesTrAmtLimit_Bank?.length) {
      this.addNewItem(
        this.trAmountLimitBank,
        'Value',
        this.f.CurrenciesTrAmtLimit_Bank,
        'Id'
      );
    }
    if (this.f.CurrenciesTrAmtLimit_DD?.length) {
      this.addNewItem(
        this.trAmountLimitDraft,
        'Value',
        this.f.CurrenciesTrAmtLimit_DD,
        'Id'
      );
    }
    if (this.f.CurrenciesTrAmtLimit_WALT?.length) {
      this.addNewItem(
        this.trAmountLimitWalt,
        'Value',
        this.f.CurrenciesTrAmtLimit_WALT,
        'Id'
      );
    }
    if (this.f.OutboundIDRequirementThreshold?.length) {
      this.addNewItem(
        this.outboundIDReqirementThreshold,
        'Value',
        this.f.OutboundIDRequirementThreshold,
        'Id'
      );
    }
  }

  removeItem() {
    if (
      this.listofCurrencies?.length &&
      this.trAmountLimits?.length &&
      this.trAmountLimitBank?.length &&
      this.trAmountLimitDraft?.length &&
      this.trAmountLimitWalt?.length &&
      this.outboundIDReqirementThreshold?.length
    ) {
      this.listofCurrencies.pop();
      this.trAmountLimits.pop();
      this.trAmountLimitBank.pop();
      this.trAmountLimitDraft.pop();
      this.trAmountLimitWalt.pop();
      this.outboundIDReqirementThreshold.pop();
      this.manageCountryForm.patchValue({
        CurrenciesTrAmtLimit: '',
        CurrenciesTrAmtLimit_Bank: '',
        CurrenciesTrAmtLimit_DD: '',
        CurrenciesTrAmtLimit_WALT: '',
        // CurrenciesOutboundIDReqirementThreshold: [],
      });
    }
  }

  addCountry() {
    if (!this.manageCountryForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (this.f.CurrenciesTrAmtLimit == '' && this.trAmountLimits.length == 0) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      this.f.CurrenciesTrAmtLimit_Bank == '' &&
      this.trAmountLimitBank.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      this.f.CurrenciesTrAmtLimit_DD == '' &&
      this.trAmountLimitDraft.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      this.f.CurrenciesTrAmtLimit_WALT == '' &&
      this.trAmountLimitWalt.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (
      this.f.OutboundIDRequirementThreshold == '' &&
      this.outboundIDReqirementThreshold.length == 0
    ) {
      alert('Please Fill Required Fields!');
      return;
    }
    if (this.f.BaseCurrencyID == 'None' && this.type == 'add') {
      alert('Please Select Currency.');
      return;
    }
    if (this.f.AllowedDomesticTransactionType == 'None') {
      alert('Please Select Allowed Domestic Transaction Type.');
      return;
    }
    if (this.f.AllowedInternationalTransactionType == 'None') {
      alert('Please Select Allowed International Transaction Type.');
      return;
    }

    // if (this.f.OutboundIDRequirementThreshold < this.f.CurrenciesTrAmtLimit) {
    //   alert('Should be less than OutboundThreshold PerTransaction (In Amount)');
    //   return;
    // }

    if (!this.f.IsAsian) {
      alert('Please Select IsAsian.');
      return;
    }

    let payload;
    payload = {
      CountryID: 0,
      BaseCurrencyID: this.f.BaseCurrencyID,
      CountryName: this.f.CountryName,
      CountryShortName: this.f.CountryShortName,
      AllowedInternationalTransactionType:
        this.f.AllowedInternationalTransactionType,
      AllowedDomesticTransactionType: this.f.AllowedDomesticTransactionType,
      TotalInboundThresholdPerYearPerPerson: Number(
        this.f.TotalInboundThresholdPerYearPerPerson
      ),
      MaximumTransactionsPerYearPerPersonToOneBeneficiary: Number(
        this.f.MaximumTransactionsPerYearPerPersonToOneBeneficiary
      ),
      TransactionYearStartDate: this.f.TransactionYearStartDate,
      TransactionYearEndDate: this.f.TransactionYearEndDate,
      OutboundIDRequirementThreshold: Number(
        this.f.OutboundIDRequirementThreshold
      ),
      OutboundThresholdPerTransaction: 0,
      OutboundThresholdPerMonthTransaction: Number(
        this.f.OutboundThresholdPerMonthTransaction
      ),
      OutboundThresholdPerDayTransaction: Number(
        this.f.OutboundThresholdPerDayTransaction
      ),
      IsAsian: this.f.IsAsian ? this.f.IsAsian : true,
      Active: true,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy:getCacheData('G_UserID'),
      Currencies: this.listofCurrencies,
      CurrenciesTrAmtLimit: this.trAmountLimits,
      CurrenciesTrAmtLimit_Bank: this.trAmountLimitBank,
      CurrenciesTrAmtLimit_DD: this.trAmountLimitDraft,
      CurrenciesTrAmtLimit_WALT: this.trAmountLimitWalt,
      CurrenciesOutboundIDReqirementThreshold:
        this.outboundIDReqirementThreshold,
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        CountryID: this.f.CountryID,
        BaseCurrencyID: this.f.UpdatedBaseCurrencyId,
      };
    }

    const serviceMethod = this.type === 'add' ? 'addCountry' : 'updateCountry';

    this.countryService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.getAllCountries();
        this.modalService.hide();
        const message = this.type === 'edit' ? 'updated' : 'added';
        alert(res);
      },
      error: (err) => {
        this.modalService.hide();
        // alert(err?.error?.message || 'Something went wrong');
        this.toastrSvc.error(err?.error?.message)
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editCountry(country: any) {
    if (country?.ID) {
      this.getCountryById(country?.ID);
    }
  }

  getCountryById(countryId = 0) {
    this.countryService
      .getCountryById(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.listofCurrencies = res?.[0]?.Currencies;
          this.trAmountLimits = res?.[0]?.CurrenciesTrAmtLimit;
          this.trAmountLimitBank = res?.[0]?.CurrenciesTrAmtLimit_Bank;
          this.trAmountLimitDraft = res?.[0]?.CurrenciesTrAmtLimit_DD;
          this.trAmountLimitWalt = res?.[0]?.CurrenciesTrAmtLimit_WALT;
          this.outboundIDReqirementThreshold =
            res?.[0]?.CurrenciesOutboundIDReqirementThreshold;

          // Convert string to Date object
          const trnsactionStartObject = new Date(
            res?.[0]?.TransactionYearStartDate
          );

          // Convert string to Date object
          const trnsactionEndObject = new Date(
            res?.[0]?.TransactionYearEndDate
          );

          // Format the date as a string recognized by the input type date
          const transactionStartDate = this.formatDateForInput(
            trnsactionStartObject
          );

          // Format the date as a string recognized by the input type date
          const transactionEndDate =
            this.formatDateForInput(trnsactionEndObject);

          const currencyId = this.currencies.find((c) => c.Currency_Id == res?.[0]?.BaseCurrencyID)?.Currency_Id;

          this.manageCountryForm.patchValue({
            CountryID: res?.[0]?.CountryID,
            BaseCurrencyID: 'None',
            UpdatedBaseCurrencyId: currencyId,
            CountryName: res?.[0]?.CountryName,
            CountryShortName: res?.[0]?.CountryShortName,
            AllowedInternationalTransactionType:
              res?.[0]?.AllowedInternationalTransactionType,
            AllowedDomesticTransactionType:
              res?.[0]?.AllowedDomesticTransactionType,
            TotalInboundThresholdPerYearPerPerson:
              res?.[0]?.TotalInboundThresholdPerYearPerPerson,
            MaximumTransactionsPerYearPerPersonToOneBeneficiary:
              res?.[0]?.MaximumTransactionsPerYearPerPersonToOneBeneficery,
            TransactionYearStartDate: transactionStartDate,
            TransactionYearEndDate: transactionEndDate,
            OutboundIDRequirementThreshold:
              res?.[0]?.OutboundIDRequirementThreshold,
            OutboundThresholdPerMonthTransaction:
              res?.[0]?.OutboundThresholdPerMonthTransaction,
            OutboundThresholdPerDayTransaction:
              res?.[0]?.OutboundThresholdPerDayTransaction,
            IsAsian: res?.[0]?.IsAsian,
            CurrenciesTrAmtLimit:'',
            CurrenciesTrAmtLimit_Bank:'',
            CurrenciesTrAmtLimit_DD:'',
            CurrenciesTrAmtLimit_WALT:''
          });
          this.openCountryModal('edit');
        },
        error: (err) => {
          // alert(err?.error?.message || 'Something went wrong');
          this.toastrSvc.error(err?.error?.message)

        },
        complete: () => { },
      });
  }

  formatDateForInput(date: Date): string {
    return date.toISOString().split('T')[0];
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  replaceSpacesWithUnderscores(obj: any): any {
    const modifiedObj = {};
    for (const key of Object.keys(obj)) {
      const modifiedKey = key.replace(/ /g, '_'); // Replace all spaces with underscores
      modifiedObj[modifiedKey] = obj[key];
    }
    return modifiedObj;
  }

  viewCountry(country: any) {
    console.log(country)
    if (country?.ID) {
      // this.getCountryById(country?.ID); this.countryService
      this.countryService.viewCountry(country?.ID, 2).subscribe({
        next: (res: any) => {
          // Apply the transformation to each object in the array
          const modifiedArray = res.map(this.replaceSpacesWithUnderscores);

          this.manageCountryForm.patchValue({
            CountryID: modifiedArray?.[0]?.ID,
            CountryName: modifiedArray?.[0]?.Country_Name,
            CountryShortName: modifiedArray?.[0]?.Country_Shortname,
            CurrencyName: modifiedArray?.[0]?.Currency,
            InternationalTransactionType:
              modifiedArray?.[0]?.International_Transaction_Type,
            DomesticTransactionType:
              modifiedArray?.[0]?.Domestic_Transaction_Type,
            TotalInboundThresholdPerYearPerPerson:
              modifiedArray?.[0]?.Inbound_Threshold_Per_Year_Per_Person,
            MaximumTransactionsPerYearPerPersonToOneBeneficiary:
              modifiedArray?.[0]
                ?.Maximum_Trans_Per_Year_Per_Person_To_OneBeneficery,
            TransactionYearStartDate:
              modifiedArray?.[0]?.Transaction_Start_Date,
            TransactionYearEndDate: modifiedArray?.[0]?.Transaction_End_Date,
            OutboundIdRequirement: modifiedArray?.[0]?.Outbound_ID_Reqirement,
            OutboundThresholdPerTransaction:
              modifiedArray?.[0]?.Outbound_Threshold_Per_Transaction,
            OutboundThresholdPerMonthTransaction:
              modifiedArray?.[0]?.Outbound_Threshold_Per_Month_Transaction,
            OutboundThresholdPerDayTransaction:
              modifiedArray?.[0]?.Outbound_Threshold_Per_Day_Transaction,
            Asian: modifiedArray?.[0]?.Is_Asian,
            Approved: modifiedArray?.[0]?.Approved,
          });
          this.openCountryModal('view');
        },
        error: (err) => {
          // alert(err?.error?.message || 'Something went wrong');
          this.toastrSvc.error(err?.error?.message)

        },
        complete: () => { },
      });
    }
  }

  approve() {
    if (this.f.CountryID) {
      this.countryService
        .approveCountry(
          getCacheData('admin_token'),
          getCacheData('username'),
          this.f.CountryID,
          2
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
          },
          error: (err) => {
            // alert(err?.error?.message || 'Something went wrong');
            this.toastrSvc.error(err?.error?.message)

            this.modalService.hide();
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  transformToUpperCase(event: Event, controlName: string): void {
    const input = event.target as HTMLInputElement;
    input.value = input.value.toUpperCase();
    this.manageCountryForm.get(controlName)?.setValue(input.value);
  }
  
}
