import { Component, TemplateRef, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { EditTransactionService } from '../services/edit-transaction.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { EditTransactionDetailsService } from '../edit-transaction-details.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';

@Component({
  selector: 'app-edit-transaction-detail',
  templateUrl: './edit-transaction-detail.component.html',
  styleUrls: ['./edit-transaction-detail.component.css'],
})
export class EditTransactionDetailComponent {
  @ViewChild('viewBankDetails') viewBankDetailsModal;
  bsModalRef: BsModalRef;

  transactionDetails: any = [];
  transactionStatus: any = [];
  destinationCity: any = [];
  recommendedAgent: any = [];
  idTypes: any = [];
  nationalities: any = [];
  branchId: any = '';
  appId: any = '';
  customerNationality: any = [];
  beneficiaryNationality: any = [];

  editTransactionForm = new FormGroup({
    TransactionNo: new FormControl('', Validators.required),
    txtRemark: new FormControl('', Validators.required),
    Beneficiary_Name_F: new FormControl('', Validators.required),
    Beneficiary_Name_L: new FormControl('', Validators.required),
    BAddress: new FormControl('', Validators.required),
    BMobile: new FormControl('', Validators.required),
    Address: new FormControl('', Validators.required),
    Mobile: new FormControl('', Validators.required),

    ddlDestinationCity: new FormControl(''),
    ddlRecommendedAgent: new FormControl(''),
    ddlTransactionStatus: new FormControl(''),
    hidLocId: new FormControl(''),
    drpIDType: new FormControl(''),
    ExpDt: new FormControl(''),
    DOB: new FormControl(''),
    IDNumber: new FormControl(''),
    BankName: new FormControl(''),
    BankBranch: new FormControl(''),
    BankAccountNo: new FormControl(''),
    txtBankCode: new FormControl(''),
    Customer_Name_F: new FormControl(''),
    Customer_Name_L: new FormControl(''),
    BPhone: new FormControl(''),
    HiddenMOPID: new FormControl(''),
    hidPayOutCountry: new FormControl(''),
    DestinationCountry: new FormControl(''),
    hidPayOutCCY: new FormControl(''),
    DestinationPayementMode: new FormControl(''),
    RecommendedAgent: new FormControl(''),
    AgentAddress: new FormControl(''),
    PurposeofTransfer: new FormControl(''),
    txtPurposeSpecify: new FormControl(''),
    HidTrType: new FormControl(''),
    hidTr_Display: new FormControl(''),
    AgentID: new FormControl(''),
    SubGroupID: new FormControl(''),
    ExchangeRate: new FormControl(''),
    PayInAmount: new FormControl(''),
    PayOutAmount: new FormControl(''),
    _PayOutAmount: new FormControl(''),
    _PayInAmount: new FormControl(''),
    lblIDRequired: new FormControl(''),
    CommissionCharge: new FormControl(''),
    TotalPayable: new FormControl(''),
    lblPayInCurrency: new FormControl(''),
    lblPayOutCurrency: new FormControl(''),
    lblCommissionCurrency: new FormControl(''),
    lblPaybleCurrency: new FormControl(''),
    TrDate: new FormControl(null),
    hidCust_Id: new FormControl(''),
    Phone: new FormControl(''),
    NationalityID: new FormControl('None'),
    HiddenCustomerDOB: new FormControl(''),
    Relation: new FormControl(''),
    txtPlaceOfBirth: new FormControl(''),
    BefZipCode: new FormControl(''),
    lblSuspected: new FormControl(true),
    hidBene_Id: new FormControl(''),
    BNationalityID: new FormControl(''),
    MessageToBen: new FormControl(''),
    MessageToAgent: new FormControl(''),
    LandMark: new FormControl(''),
    IsArebic: new FormControl(true),
    CourierAddress: new FormControl(''),
    CourierRemark: new FormControl(''),
    BankAddress: new FormControl(''),
    DDNumber: new FormControl(''),
    DDBank: new FormControl(''),
    DDBranch: new FormControl(''),
    DDBankAddress: new FormControl(''),
    UserID: new FormControl(''),
    TrTypeID: new FormControl(''),
    hidAppID: new FormControl(''),
    hidBranch: new FormControl(''),
    hidReceiveBranchID: new FormControl(''),
    HiddenExpDt: new FormControl(''),
  });

  // bank details modal
  paymentMethod: string = 'neft';

  constructor(
    private EditTrSvc: EditTransactionService,
    private EditTransactionDetailsSvc: EditTransactionDetailsService,
    private route: ActivatedRoute,
    private modalService: BsModalService,
    private router: Router
  ) {}

  ngOnInit(): void {
    this.branchId = this.route.snapshot.params['Branch'];
    this.appId = this.route.snapshot.params['AppId'];
    if (this.branchId && this.appId) {
      this.getTransactionDetails(this.branchId, this.appId);
      console.log(
        'Date Processing:',
        this.formatDateForInput(this.transactionDetails)
      );
    }
    this.getCustomerNationality();
    this.getBeneficiaryNationality();
  }

  isValidDate(dateString: string): boolean {
    const datePattern = /^(0[1-9]|[12][0-9]|3[01])\/(0[1-9]|1[0-2])\/\d{4}$/;

    if (!datePattern.test(dateString)) {
      return false;
    }

    const [day, month, year] = dateString.split('/').map(Number);
    const dateObj = new Date(year, month - 1, day);

    return (
      dateObj.getFullYear() === year &&
      dateObj.getMonth() === month - 1 &&
      dateObj.getDate() === day
    );
  }

  getTransactionDetails(branchId: any = 0, appId: any = 0) {
    this.EditTrSvc.getTransactionDetails(branchId, appId).subscribe({
      next: (res: any) => {
        console.log(res?.[0]?.TrDate);
        this.transactionDetails = res[0];
        this.transactionStatus = res?.[0]?.status;
        this.destinationCity = res?.[0]?.city;
        this.recommendedAgent = res?.[0]?.getAgentLists;
        console.log('Original date from the server:', res[0].TrDate);
        const transactionDate = this.convertDateToISOFormat(res[0].TrDate);
        console.log(transactionDate);
        const { DOB, BNationalityID, NationalityID } = res?.[0] || {};

        const date = this.isValidDate(DOB);
        this.editTransactionForm.patchValue({
          DOB: date ? this.convertDateToISOFormat(DOB) : DOB,
          BNationalityID: BNationalityID || '-Select Nationality-',
          NationalityID: NationalityID || '-Select Nationality-',
        });

        this.idTypes = res?.[0]?.iDTypes;
        if (res?.[0]?.city?.length) {
          this.editTransactionForm.patchValue({
            ddlDestinationCity: res?.[0]?.selectedCityID,
          });
        }
        this.editTransactionForm.patchValue({
          TransactionNo: res?.[0]?.TansactionNo,
          DestinationCountry: res?.[0]?.DestinationCountry,
          ddlTransactionStatus: res?.[0]?.ddlTransactionStatusID,
          DestinationPayementMode: res?.[0]?.DestinationPayementMode,
          // ddlDestinationCity: res?.[0]?.selectedCityID,
          ddlRecommendedAgent: res?.[0]?.ddlRecommendedAgent,
          AgentAddress: res?.[0]?.AgentAddress,
          PurposeofTransfer: res?.[0]?.PurposeofTransfer,
          txtPurposeSpecify: res?.[0]?.txtPurposeSpecify,
          TrDate: transactionDate,
          ExchangeRate: res?.[0]?.ExchangeRate,
          PayInAmount: res?.[0]?.PayInAmount,
          PayOutAmount: res?.[0]?.PayOutAmount,
          CommissionCharge: res?.[0]?.CommissionCharge,
          TotalPayable: res?.[0]?.TotalPayable,
          drpIDType: res?.[0]?.drpIDType,
          IDNumber: res?.[0]?.IDNumber,
          ExpDt: res?.[0]?.ExpDt,
          Customer_Name_F: res?.[0]?.Customer_Name_F,
          Customer_Name_L: res?.[0]?.Customer_Name_L,
          Address: res?.[0]?.Address,
          Mobile: res?.[0]?.Mobile,
          Phone: res?.[0]?.Phone,
          // NationalityID: res?.[0]?.NationalityID,
          // DOB: res?.[0]?.DOB,
          txtPlaceOfBirth: res?.[0]?.txtPlaceOfBirth,
          Relation: res?.[0]?.Relation,
          Beneficiary_Name_F: res?.[0]?.Beneficiary_Name_F,
          Beneficiary_Name_L: res?.[0]?.Beneficiary_Name_L,
          BAddress: res?.[0]?.BAddress,
          BMobile: res?.[0]?.BMobile,
          BPhone: res?.[0]?.BPhone,
          // BNationalityID: res?.[0]?.BNationalityID,
          MessageToBen: res?.[0]?.MessageToBen,
          MessageToAgent: res?.[0]?.MessageToAgent,
          LandMark: res?.[0]?.LandMark,
          BefZipCode: res?.[0]?.BefZipCode,
          BankName: res?.[0]?.BankName,
          BankBranch: res?.[0]?.BankBranch,
          BankAddress: res?.[0]?.BankAddress,
          BankAccountNo: res?.[0]?.BankAccountNo,
          txtBankCode: res?.[0]?.txtBankCode,
          hidLocId: res?.[0]?.hidLocId,
          txtRemark: res?.[0]?.txtRemark,
        });
      },
      error: () => {
        this.transactionDetails = [];
      },
      complete: () => {},
    });
  }

  formatDateForInput(dateString: string): string {
    const date = new Date(dateString);
    if (isNaN(date.getTime())) {
      // console.warn('Invalid date provided:', dateString);
      return ''; // Return an empty string if the date is invalid
    }
    return `${date.getFullYear()}-${(date.getMonth() + 1)
      .toString()
      .padStart(2, '0')}-${date.getDate().toString().padStart(2, '0')}`;
  }

  convertDateToISOFormat(d: string): string {
    if (!d || !d.includes('/')) {
      console.warn('Invalid or unexpected date format:', d);
      return ''; // Safely return an empty string if the date format is unexpected
    }
    const [day, month, year] = d.split('/');
    return `${year}-${month.padStart(2, '0')}-${day.padStart(2, '0')}`;
  }

  formatDate(dateString: string): string {
    const formattedInput = this.formatDateForInput(dateString);
    const date = new Date(formattedInput);
    if (isNaN(date.getTime())) {
      // console.warn('Invalid date conversion for:', formattedInput);
      return '';
    }
    // Ensuring the day and month are correctly formatted with leading zeros if necessary
    return `${date.getDate().toString().padStart(2, '0')}/${(
      date.getMonth() + 1
    )
      .toString()
      .padStart(2, '0')}/${date.getFullYear()}`;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  get f() {
    return this.editTransactionForm.value;
  }

  updateTransfer() {
    if (!this.editTransactionForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    // if (this.f.DOB == 'dd/MM/yyyy') {
    //   alert('Please Fill DOB!');
    //   return;
    // }

    if (this.transactionDetails?.HidTrType == '1') {
      if (
        !this.f.BankName ||
        !this.f.BankAccountNo ||
        !this.f.BankBranch ||
        !this.f.txtBankCode
      ) {
        alert('Please Fill Required Fields!');
        return;
      }
    }

    if (this.f.DOB == 'dd/MM/yyyy') {
      this.f.DOB = '';
    } else {
      this.f.DOB = this.formatDate(this.f.DOB);
    }

    if (this.f.ExpDt == 'dd/MM/yyyy') {
      this.f.ExpDt = '';
    } else {
      this.f.ExpDt = this.formatDate(this.f.ExpDt);
    }

    const hidReceiveBranchID = this.recommendedAgent.find(
      (a) => a.ddlRecommendedAgentID == this.f.ddlRecommendedAgent
    )?.hidReceiveBranchID;

    let payload;
    payload = {
      Address: this.f.Address,
      Mobile: this.f.Mobile,
      Phone: this.f.Phone ? this.f.Phone : '',
      Beneficiary_Name_F: this.f.Beneficiary_Name_F,
      Beneficiary_Name_L: this.f.Beneficiary_Name_L,
      BAddress: this.f.BAddress,
      BMobile: this.f.BMobile,
      BPhone: this.f.BPhone ? this.f.BPhone : '',
      BNationalityID:
        this.f.BNationalityID == '-Select Nationality-'
          ? '0'
          : this.f.BNationalityID,
      Ex_Branch_ID: this.branchId,
      App_ID: this.appId,
      TrStatusID: this.f.ddlTransactionStatus,
      PayOutCityID: this.f.ddlDestinationCity ? this.f.ddlDestinationCity : '0',
      PayOutLocationID: this.f.hidLocId,
      PayOutAgent: this.f.ddlRecommendedAgent,
      BDO_Branch_ID: hidReceiveBranchID ? hidReceiveBranchID : '0',
      Remark: this.f.txtRemark,
      TrTypeID: this.transactionDetails?.HidTrType,
      Type_ID: this.f.drpIDType,
      IDNumber: this.f.IDNumber,
      IDExpDt: this.f.ExpDt,
      // IDExpDt:  '03/03/2024',
      EnteredBy: '1001000',
      DOB: this.f.DOB,
      // DOB: '11/11/1980',
      BankAccountNo: this.f.BankAccountNo,
      BankName: this.f.BankName,
      BankCode: this.f.txtBankCode,
      BankBranch: this.f.BankBranch,
      BankAddress: this.f.BankAddress,
      TransactionNo: this.f.TransactionNo,
    };

    console.log('payload: ', payload);
    this.EditTrSvc.updateTransfer(payload).subscribe({
      next: (res: any) => {
        if (res) {
          alert('Record updated successfully.');
          return;
        }
      },
      error: (err) => {
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {},
    });
  }
  goBack(): void {
    // Implement logic to navigate back to the previous page or route
    this.router.navigate(['/panel/edit-transaction']); // Example navigation
  }

  getCustomerNationality() {
    this.EditTransactionDetailsSvc.getCustomerNationality().subscribe({
      next: (res: any) => {
        this.customerNationality = res;
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  getBeneficiaryNationality() {
    this.EditTransactionDetailsSvc.getBeneficiaryNationality().subscribe({
      next: (res: any) => {
        this.beneficiaryNationality = res;
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  openBankDetailModal() {
    this.openModal(this.viewBankDetailsModal);
  }

  search() {}
}
