import { Component, TemplateRef, ViewChild } from '@angular/core';
import { EditTransactionService } from '../services/edit-transaction.service';
import { ManageCountryPermissionService } from '../services/manage-country-permission.service';
import { Router } from '@angular/router';
import { AuthService } from '../services/auth.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';

@Component({
  selector: 'app-edit-transaction',
  templateUrl: './edit-transaction.component.html',
  styleUrls: ['./edit-transaction.component.css'],
})
export class EditTransactionComponent {
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;

  AgentID: any;
  AgentUserList: any;
  AgentBranchList: any;
  PayOutCurrencyList: any;
  PayinCurrencyList: any;
  baseCountries: any = -1;
  searchByType: any = 1;
  isLoading: boolean = false;

  status: any = [
    {
      Id: -1,
      Value: 'ALL',
    },
  ];
  TransferTypeList: any = [];
  payinCountryList: any = -1;
  viewLast: any = 10;
  statusId: any = -1;
  data: any = [];
  startDate: any;
  endDate: any;
  transactionNo: any;
  customerNameFirst: any;
  customerNameLast: any;
  beneficiaryNameFirst: any;
  beneficiaryNameLast: any;
  AgentList: any;
  selectedUserName: any = -1;
  receiptData: any = {};

  constructor(
    private EditTrSvc: EditTransactionService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private modalService: BsModalService,
    private router: Router,
    private authSvc: AuthService
  ) {}
  ngOnInit() {
    this.getSerchStatusList();
    this.getReceivingCountryPermission();
    this.getpayinCountryList();
    this.getTransferTypeList();
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  getSerchStatusList() {
    this.EditTrSvc.getSerchStatusList().subscribe({
      next: (res: any) => {
        this.status = this.status.concat(
          res.map((t) => {
            return {
              ID: t.TrStatusID,
              Value: t.Tr_Status_Name,
            };
          })
        );
      },
    });
  }

  getReceivingCountryPermission() {
    this.EditTrSvc.getPayOutCountry().subscribe({
      next: (res: any) => {
        this.baseCountries = res;
      },
    });
  }

  getpayinCountryList() {
    this.EditTrSvc.getPayinCountryList().subscribe({
      next: (res: any) => {
        this.payinCountryList = res;
      },
    });
  }

  onSelectDestCountry(countryId) {
    this.getPayinCurrencyList(countryId.target.value);
  }

  getPayinCurrencyList(countryId) {
    this.EditTrSvc.getPayinCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.PayinCurrencyList = res;
      },
    });
  }

  onSelectSendCountry(countryId) {
    this.getPayOutCurrencyList(countryId);
    this.getAgentList(countryId);
  }

  getAgentList(countryId) {
    this.EditTrSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.AgentList = res;
      },
    });
  }

  onSelectAgent(AgentID) {
    this.AgentID = AgentID;
    this.getAgentBranch(AgentID);
  }

  getAgentBranch(agentID) {
    this.EditTrSvc.getAgentBranch(agentID).subscribe({
      next: (res: any) => {
        this.AgentBranchList = res;
      },
    });
  }

  onSelectAgentBranch(AgentBranchID) {
    this.getAgentUser(AgentBranchID);
  }

  getAgentUser(AgentBranchID: any) {
    this.EditTrSvc.getAgentUser(this.AgentID, AgentBranchID).subscribe({
      next: (res: any) => {
        this.AgentUserList = res;
      },
    });
  }

  getPayOutCurrencyList(countryId) {
    this.EditTrSvc.getPayOutCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.PayOutCurrencyList = res;
      },
    });
  }

  getTransferTypeList() {
    this.EditTrSvc.getTransferTypeList().subscribe({
      next: (res: any) => {
        this.TransferTypeList = res;
      },
    });
  }

  getFillGrid() {
    if (this.searchByType === 1 && !this.transactionNo) {
      alert('Please Fill Transaction Number.');
      return;
    }
    this.isLoading = true;
    let payload: any = {};

    if (this.searchByType === 2) {
      payload = {
        SearchByType: this.searchByType,
        TopRecord: this.viewLast,
        Agent_ID: 0,
        PayOutCountry: -1,
        PayOutCurrency: -1,
        SubType: -1,
        // SRTRNo: '',
        TrStatusID: this.statusId,
        TrStatusList: this.status,
        TrTypeID: -1,
        PayInCountry: -1,
        PayInCurrency: -1,
        UserID: '-1',
        AgentBranchId: -1,
      };
    }
    if (this.searchByType === 1) {
      payload = {
        SearchByType: this.searchByType,
        TopRecord: this.viewLast,
        Agent_ID: 0,
        PayOutCountry: -1,
        PayOutCurrency: -1,
        SubType: -1,
        SRTRNo: this.transactionNo,
        TrStatusID: this.statusId,
        TrStatusList: this.status,
        TrTypeID: -1,
        PayInCountry: -1,
        PayInCurrency: -1,
        UserID: '-1',
        AgentBranchId: -1,
      };
    }
    if (this.searchByType === 3) {
      payload = {
        SearchByType: this.searchByType,
        TopRecord: this.viewLast,
        Agent_ID: 0,
        PayOutCountry: -1,
        PayOutCurrency: -1,
        SubType: -1,
        SRTRNo: this.transactionNo,
        TrStatusID: this.statusId,
        TrStatusList: this.status,
        TrTypeID: -1,
        StartDate: this.startDate,
        EndDate: this.endDate,
        PayInCountry: -1,
        PayInCurrency: -1,
        UserID: '-1',
        AgentBranchId: -1,
        Beneficiary_Name_F: this.beneficiaryNameFirst
          ? this.beneficiaryNameFirst
          : '',
        Beneficiary_Name_L: this.beneficiaryNameLast
          ? this.beneficiaryNameLast
          : '',
        Customer_Name_F: this.customerNameFirst ? this.customerNameFirst : '',
        Customer_Name_L: this.customerNameLast ? this.customerNameLast : '',
      };
      console.log('this.startDate: ', this.startDate);
    }
    this.EditTrSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        this.data = res;
        if (this.data?.length == 0) {
          alert('No data found.');
        }
      },
      error: (err: any) => {
        this.isLoading = false;
        alert('No data found.');
      },
      complete: () => {
        // Set isLoading back to false when the call is complete
        this.isLoading = false;
      },
    });
  }

  ViewLast($event) {
    this.viewLast = $event.target.value;
  }

  SearchType(event) {
    this.searchByType = Number(event.target.value);
  }

  onChangeStatus(event: any) {
    this.statusId = !isNaN(Number(event.target.value))
      ? Number(event.target.value)
      : -1;
  }

  editTransaction(transaction: any) {
    if (transaction?.App_ID && transaction?.Ex_Branch_ID) {
      this.router.navigate([
        '/panel/edit-transaction-detail',
        transaction?.Ex_Branch_ID,
        transaction?.App_ID,
      ]);
    }
  }

  approved() {
    const updatedTransactions = this.data
      .filter((con) => con.isChecked)
      .map((con) => ({
        IsChkCheked: true,
        hid: con?.hid,
      }));
    if (updatedTransactions?.length) {
      const payload = {
        UserID: '1',
        GroupID: 0,
        HidValue: updatedTransactions,
      };
      this.EditTrSvc.statusApproved(payload).subscribe({
        next: (res: any) => {
          if (res) {
            this.data = [];
            this.searchByType = 1;
            this.clearCriteria();
            alert('Status has been approved.');
          }
        },
        error: (err) => {
          alert(err?.error?.message || 'Something went wrong');
        },
        complete: () => {},
      });
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  clearCriteria() {
    this.transactionNo = '';
    // this.startDate = '';
    // this.endDate = '';
    this.beneficiaryNameFirst = '';
    this.beneficiaryNameLast = '';
    this.customerNameFirst = '';
    this.customerNameLast = '';
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.SRTRNo;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }
}
