import { Component, TemplateRef, ViewChild } from '@angular/core';
import { ManagePremiumMarginsService } from '../services/manage-premium-margins.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'app-manage-premium-margins',
  templateUrl: './manage-premium-margins.component.html',
  styleUrls: ['./manage-premium-margins.component.css'],
})
export class ManagePremiumMarginsComponent {
  @ViewChild('premiumMargin') premiumMarginModal;
  bsModalRef: BsModalRef;

  sendCountryList: any = [];
  selectedSendCountry: any;
  receiveCountryList: any = [];
  selectedReceiveCountry: any;
  sendCountryAddList: any = [];
  sendingAgentList: any = [];
  agentCurrencyList: any = [];
  receiveCountryAddList: any = [];
  receivingAgentList: any = [];
  receiveCurrencyList: any = [];
  modeofPaymentList: any = [];

  premiumMargins: any = [];
  type: any = 'add';

  premiumMarginForm = new FormGroup({
    Id: new FormControl(),
    PayInCountryId: new FormControl('None', Validators.required),
    Agent_ID: new FormControl('None', Validators.required),
    Agent_CurrencyID: new FormControl('None', Validators.required),
    Target_CountryID: new FormControl('None', Validators.required),
    TargetAgent_ID: new FormControl('None', Validators.required),
    Target_CurrencyID: new FormControl('None', Validators.required),
    ModeOfPayment_ID: new FormControl(
      [
        {
          ModeOfPayment_ID: 0,
          ModeOfPayment_Name: 'None',
        },
      ],
      Validators.required
    ),
    Margin_Rate: new FormControl('', Validators.required),
  });

  constructor(
    private premiumMarginService: ManagePremiumMarginsService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendCountryList();
    this.getSendCountryAddList();
  }

  getSendCountryList() {
    this.premiumMarginService.getSendCountryList().subscribe({
      next: (res: any) => {
        this.sendCountryList = res;
        this.selectedSendCountry = res?.[0]?.CountryID;
        this.getReceiveCountryList(this.selectedSendCountry);
      },
      error: (err) => {
        this.sendCountryList = [];
      },
      complete: () => {},
    });
  }

  onSendCountryChange(countryId: any) {
    if (countryId) {
      this.getReceiveCountryList(countryId);
    }
  }

  getReceiveCountryList(sendCountryId: any = 0) {
    this.premiumMarginService.getReceiveCountryList(sendCountryId).subscribe({
      next: (res: any) => {
        this.receiveCountryList = res;
        this.selectedReceiveCountry = res?.[0]?.CountryID;
      },
      error: (err) => {
        this.receiveCountryList = [];
      },
      complete: () => {},
    });
  }

  filter() {
    if (this.selectedSendCountry && this.selectedReceiveCountry) {
      this.premiumMarginService
        .getFillGrid(this.selectedSendCountry, this.selectedReceiveCountry)
        .subscribe({
          next: (res: any) => {
            if (res?.length == 0) {
              alert('No data found.');
              return;
            }
            this.premiumMargins = res;
          },
          error: (err) => {
            this.premiumMargins = [];
          },
          complete: () => {},
        });
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openPremiumMarginModal(type: any) {
    if (type == 'add') {
      this.premiumMarginForm.reset({
        PayInCountryId: 'None',
      });
    }
    this.type = type;
    this.openModal(this.premiumMarginModal);
  }

  get f() {
    return this.premiumMarginForm.value;
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  getSendCountryAddList() {
    this.premiumMarginService.getSendCountryAddList().subscribe({
      next: (res: any) => {
        this.sendCountryAddList = res;
      },
      error: (err) => {
        this.sendCountryAddList = [];
      },
      complete: () => {},
    });
  }

  onSendCountryAddChange(countryId) {
    if (countryId) {
      this.getSendAgentList(countryId);
    }
  }

  getSendAgentList(targetCountryId: any = 0) {
    this.premiumMarginService.getSendAgentList(targetCountryId).subscribe({
      next: (res: any) => {
        this.sendingAgentList = res;
        this.premiumMarginForm.patchValue({
          Agent_ID: 'None',
        });
      },
      error: (err) => {
        this.sendingAgentList = [];
      },
      complete: () => {},
    });
  }

  onSendAgentChange(agentId) {
    if (agentId) {
      this.getAgentCurrencyList(agentId);
      this.getReceiveCountryAddList();
    }
  }

  getAgentCurrencyList(agentId: any = 0) {
    this.premiumMarginService.getAgentCurrencyList(agentId).subscribe({
      next: (res: any) => {
        this.agentCurrencyList = res;
        this.premiumMarginForm.patchValue({
          Agent_CurrencyID: 'None',
        });
      },
      error: (err) => {
        this.agentCurrencyList = [];
      },
      complete: () => {},
    });
  }

  getReceiveCountryAddList() {
    this.premiumMarginService.getReceiveCountryAddList().subscribe({
      next: (res: any) => {
        this.receiveCountryAddList = res;
        this.premiumMarginForm.patchValue({
          Target_CountryID: 'None',
        });
      },
      error: (err) => {
        this.receiveCountryAddList = [];
      },
      complete: () => {},
    });
  }

  onReceiveCountryAddChange(countryId) {
    if (countryId) {
      this.getReceivingAgentList(countryId);
    }
  }

  getReceivingAgentList(receiveCountryId: any = 0) {
    this.premiumMarginService
      .getReceivingAgentList(receiveCountryId)
      .subscribe({
        next: (res: any) => {
          this.receivingAgentList = res;
          this.premiumMarginForm.patchValue({
            TargetAgent_ID: 'None',
          });
        },
        error: (err) => {
          this.receivingAgentList = [];
        },
        complete: () => {},
      });
  }

  onReceivingAgentChange(agentId) {
    if (agentId) {
      this.getReceiveCurrencyList(agentId);
      this.getModeofPaymentList();
    }
  }

  getReceiveCurrencyList(receiveAgentId: any = 0) {
    this.premiumMarginService.getReceiveCurrencyList(receiveAgentId).subscribe({
      next: (res: any) => {
        this.receiveCurrencyList = res;
        this.premiumMarginForm.patchValue({
          Target_CurrencyID: 'None',
        });
      },
      error: (err) => {
        this.receiveCurrencyList = [];
      },
      complete: () => {},
    });
  }

  getModeofPaymentList() {
    this.premiumMarginService.getModeofPaymentList().subscribe({
      next: (res: any) => {
        this.modeofPaymentList = res;
      },
      error: (err) => {
        this.modeofPaymentList = [];
      },
      complete: () => {},
    });
  }

  addPremiumMargin() {
    if (!this.premiumMarginForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.PayInCountryId == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    if (this.f.Agent_ID == 'None') {
      alert('Please Select Sending Agent!');
      return;
    }

    if (this.f.Agent_CurrencyID == 'None') {
      alert('Please Select Agent Currency!');
      return;
    }

    if (this.f.Target_CountryID == 'None') {
      alert('Please Select Receive Country!');
      return;
    }

    if (this.f.TargetAgent_ID == 'None') {
      alert('Please Select Receiving Agent!');
      return;
    }

    if (this.f.Target_CurrencyID == 'None') {
      alert('Please Select Receiving Currency!');
      return;
    }

    const modeofPaymentName = this.modeofPaymentList.find(
      (m) => m.ModeOfPayment_ID == this.f.ModeOfPayment_ID
    )?.ModeOfPayment_Name;

    let payload;
    payload = {
      PayInCountryId: this.f.PayInCountryId,
      Agent_ID: this.f.Agent_ID,
      Agent_CurrencyID: this.f.Agent_CurrencyID,
      Target_CountryID: this.f.Target_CountryID,
      TargetAgent_ID: this.f.TargetAgent_ID,
      Target_CurrencyID: this.f.Target_CurrencyID,
      ModeOfPayment_ID: [
        {
          ModeOfPayment_ID: this.f.ModeOfPayment_ID,
          ModeOfPayment_Name: modeofPaymentName,
        },
      ],
      Margin_Rate: Number(this.f.Margin_Rate),
      G_UserID: '1',
    };

    const serviceMethod =
      this.type === 'add' ? 'addPremiumMargin' : 'updatePremiumMargin';

    this.premiumMarginService[serviceMethod](payload).subscribe({
      next: (res: any) => {
        this.filter();
        alert('Agent wise rate variance information added successfully.');
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  resetPremiumMargin() {
    this.premiumMarginForm.reset({
      PayInCountryId: 'None',
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
