import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ManageSourceOfIncomeService } from '../services/manage-source-of-income.service';
import { ToastrService } from 'ngx-toastr';
@Component({
  selector: 'app-manage-source-of-income',
  templateUrl: './manage-source-of-income.component.html',
  styleUrls: ['./manage-source-of-income.component.css'],
})
export class ManageSourceOfIncomeComponent {
  @ViewChild('sourceOfIncome') sourceOfIncomeModal;
  bsModalRef: BsModalRef;

  sourceOfIncomeList: any = [];
  selectedIncome: any = '0';
  allSourceOfIncome: any = [];
  type: any = 'add';
  isLoading: boolean = false;

  sourceOfIncomeForm = new FormGroup({
    SourceOfIncome_ID: new FormControl(),
    SourceOfIncomeName: new FormControl('', Validators.required),
    chkIsActive: new FormControl(false),
  });

  constructor(
    private sourceOfIncomeSvc: ManageSourceOfIncomeService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit(): void {
    this.getSourceOfIncomeList();
    this.getFillGrid();
  }

  getSourceOfIncomeList() {
    this.isLoading = true;

    this.sourceOfIncomeSvc.getSourceOfIncomeList().subscribe({
      next: (res: any) => {
        this.sourceOfIncomeList = res;
      },
      error: (err) => {
        this.sourceOfIncomeList = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  getFillGrid(sourceOfIncome: any = 0) {
    this.isLoading = true;

    this.sourceOfIncomeSvc.getFillGrid(sourceOfIncome).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allSourceOfIncome = res;
        if (this.allSourceOfIncome.length) {
          this.allSourceOfIncome = this.allSourceOfIncome.flatMap(
            (item) => item.incomeDetails
          );
        }
      },
      error: (err) => {
        this.allSourceOfIncome = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  filter() {
    if (this.selectedIncome) {
      this.getFillGrid(this.selectedIncome);
    }
  }

  addSourceOfIncome() {
    if (!this.sourceOfIncomeForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    const sourceOfIncomeName = this.f.SourceOfIncomeName;
    const isActive = this.f.chkIsActive || false;
    const sourceOfIncomeId = this.f.SourceOfIncome_ID;

    const handleResponse = (message: any) => {
      this.modalService.hide();
      this.getFillGrid();
      this.getSourceOfIncomeList();
      alert(message);
    };

    const handleError = (err: any) => {
      this.modalService.hide();
      this.toastrSvc.error(err?.error?.message)
      // alert(err?.error?.message || 'Something went wrong');
    };

    const request$ =
      this.type === 'edit'
        ? this.sourceOfIncomeSvc.updateSourceOfIncome(
            sourceOfIncomeId,
            sourceOfIncomeName,
            isActive
          )
        : this.sourceOfIncomeSvc.addSourceOfIncome(
            sourceOfIncomeName,
            isActive
          );

    request$.subscribe({
      next: (response) =>
        handleResponse(
          response
        ),
      error: (err) => {
        handleError(err);
      },
      complete: () => this.modalService.hide(),
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSourceOfIncomeModal(type: any) {
    if (type == 'add') {
      this.sourceOfIncomeForm.reset();
    }
    this.type = type;
    this.openModal(this.sourceOfIncomeModal);
  }

  get f() {
    return this.sourceOfIncomeForm.value;
  }

  reset() {
    this.sourceOfIncomeForm.reset();
  }

  editSourceofIncome(sourceOfIncome: any) {
    this.sourceOfIncomeForm.patchValue({
      SourceOfIncome_ID: sourceOfIncome?.SourceOfIncome_ID,
      SourceOfIncomeName: sourceOfIncome?.SourceOfIncome,
      chkIsActive: sourceOfIncome?.IsActive,
    });
    this.openSourceOfIncomeModal('edit');
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
