import { Component } from '@angular/core';
import { MapTrMasterService } from '../services/map-tr-master.service';
import { ToastrService } from 'ngx-toastr';
@Component({
  selector: 'app-map-tr-master',
  templateUrl: './map-tr-master.component.html',
  styleUrls: ['./map-tr-master.component.css'],
})
export class MapTrMasterComponent {
  isLoading: boolean = false;

  payOutXmlList: any = [];
  selectedPayOutXml: any = '0';
  countryList: any = [];
  selectedCountry: any = '0';
  currencyList: any = [];
  selectedCurrency: any = '0';
  deliveryOptionList: any = [];
  selectedDeliveryOption: any = '0';
  receivingAgentList: any = [];
  selectedAgent: any = '0';
  statusList: any = [
    {
      id: 1,
      value: 'True',
    },
    {
      id: 2,
      value: 'False',
    },
  ];
  selectedStatus: any = '0';
  allRecord: any = [];
  trTagList: any = [];

  constructor(private mapTrMasterSvc: MapTrMasterService,private toastrSvc: ToastrService) {}

  ngOnInit(): void {
    this.getPayoutXmlList();
    this.getCountryList();
  }

  getPayoutXmlList() {
    this.mapTrMasterSvc.getPayoutXmlList().subscribe({
      next: (res: any) => {
        this.payOutXmlList = res;
      },
      error: (err) => {
        this.payOutXmlList = [];
      },
      complete: () => {},
    });
  }

  onPayoutXmlChange() {
    if (this.selectedPayOutXml != '0') {
      this.getAllRecord(this.selectedPayOutXml);
      this.getTrTagList(this.selectedPayOutXml);
    }
  }

  getCountryList() {
    this.mapTrMasterSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange() {
    if (this.selectedCountry != '0') {
      this.getCurrencyList(this.selectedCountry);
    }
  }

  getCurrencyList(countryId) {
    this.mapTrMasterSvc.getCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.currencyList = res;
      },
      error: (err) => {
        this.currencyList = [];
      },
      complete: () => {},
    });
  }

  onCurrencyChange() {
    if (this.selectedCountry != '0' && this.selectedCurrency != '0') {
      this.getDeliveryOptionList(this.selectedCountry, this.selectedCurrency);
    }
  }

  getDeliveryOptionList(countryId, currencyId) {
    this.mapTrMasterSvc.getDeliveryOptionList(countryId, currencyId).subscribe({
      next: (res: any) => {
        this.deliveryOptionList = res;
      },
      error: (err) => {
        this.deliveryOptionList = [];
      },
      complete: () => {},
    });
  }

  onDeliveryOptionChange() {
    if (
      this.selectedCountry != '0' &&
      this.selectedCurrency != '0' &&
      this.selectedDeliveryOption != '0'
    ) {
      this.getReceivingAgentList(
        this.selectedCountry,
        this.selectedCurrency,
        this.selectedDeliveryOption
      );
    }
  }

  getReceivingAgentList(countryId, currencyId, deliveryOption) {
    this.mapTrMasterSvc
      .getReceivingAgentList(countryId, currencyId, deliveryOption)
      .subscribe({
        next: (res: any) => {
          this.receivingAgentList = res;
        },
        error: (err) => {
          this.receivingAgentList = [];
        },
        complete: () => {},
      });
  }

  getAllRecord(xmlTypeId) {
    this.mapTrMasterSvc.getAllRecord(xmlTypeId).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allRecord = res.map((record: any) => ({
          ...record,
          editMode: false,
        }));
      },
      error: (err) => {
        this.allRecord = [];
      },
      complete: () => {},
    });
  }

  filter() {
    if (this.selectedPayOutXml == '0') {
      alert('Please Select Payout XmL Tag.');
      return;
    }
    if (this.selectedPayOutXml != '0') {
      this.getAllRecord(this.selectedPayOutXml);
    }
  }

  getTrTagList(payOutXmlTag) {
    this.isLoading = true;

    this.mapTrMasterSvc.getTrTagList(payOutXmlTag).subscribe({
      next: (res: any) => {
        this.isLoading = false; 

        this.trTagList = res;
      },
      error: (err) => {
        this.isLoading = false; 
        this.trTagList = [];
      },
      complete: () => {},
    });
  }

  update(trMaster: any) {
    if (trMaster) {
      this.mapTrMasterSvc
        .updateTrMaster(
          this.selectedPayOutXml,
          trMaster?.Id,
          trMaster?.TRValueID,
          trMaster?.IsActive
        )
        .subscribe({
          next: (res: any) => {
            alert(res);
            this.getAllRecord(this.selectedPayOutXml);
          },
          error: (err) => { this.toastrSvc.error(err?.error?.message)},
          complete: () => {},
        });
    }
  }

  toggleEdit(rec: any) {
    rec.editMode = true;
  }

  toggleCancel(rec: any) {
    rec.editMode = false;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
