import { Component, TemplateRef, ViewChild } from '@angular/core';
import { RelationshipManagementService } from '../services/relationship-management.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ToastrService } from 'ngx-toastr';
@Component({
  selector: 'app-relationship-management',
  templateUrl: './relationship-management.component.html',
  styleUrls: ['./relationship-management.component.css'],
})
export class RelationshipManagementComponent {
  @ViewChild('relationManagement') relationManagementModal;
  bsModalRef: BsModalRef;

  relationShipManagement: any = [];
  allRelationShipManagement: any = [];
  selectedRelation: any = '-1';
  type: any = 'add';
  isLoading: boolean = false;

  relationManagementForm = new FormGroup({
    Relation_ID: new FormControl(''),
    RelationName:  new FormControl('', [
      Validators.required,
      Validators.pattern(/^[A-Za-z ]+$/)  // Allows only letters and spaces
    ]),
    IsActive: new FormControl(false),
  });

  constructor(
    private relationService: RelationshipManagementService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit(): void {
    this.getRelationList();
  }

  getRelationList() {
    this.isLoading = true;

    this.relationService.getRelationList().subscribe({
      next: (res: any) => {
        this.relationShipManagement = res;
        this.allRelationShipManagement = res;
      },
      error: (err) => {
        this.relationShipManagement = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  addRelationManagement() {
    if (!this.relationManagementForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    let payload = {
      IsActive: this.f.IsActive,
      UserID: 0,
    };

    if (this.type === 'add') {
      payload['RelationName'] = this.f.RelationName;
    }

    if (this.type === 'edit') {
      payload['Relation_ID'] = this.f.Relation_ID;
      payload['Relation_Name'] = this.f.RelationName;
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addRelationManagement'
        : 'updateRelationManagement';

    this.relationService[serviceMethod](payload).subscribe({
      next: (res: any) => {
        this.getRelationList();
        this.modalService.hide();
        const message = this.type === 'edit' ? 'updated' : 'added';
        alert(res);
      
      },
      error: (err) => {
        this.modalService.hide();
        this.toastrSvc.error(err?.error?.message)
        // alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openRelationManagementModal(type: any) {
    if (type == 'add') {
      this.relationManagementForm.reset();
    }
    this.type = type;
    this.openModal(this.relationManagementModal);
  }

  get f() {
    return this.relationManagementForm.value;
  }

  reset() {
    this.relationManagementForm.reset();
  }

  filter() {
    this.isLoading = true;

    if (this.selectedRelation != '-1') {
      this.relationService
        .getRelationListByRelationName(this.selectedRelation)
        .subscribe({
          next: (res: any) => {
            this.isLoading = false; 
            if (res?.length == 0) {
              alert('No data found.');
              return;
            }
            this.relationShipManagement = res;
          },
          error: (err) => {
            this.isLoading = false; 
            this.relationShipManagement = [];
          },
          complete: () => {},
        });
    } else {
      this.getRelationList();
    }
  }

  editRelation(relation: any) {
    this.relationService.editRelation(relation?.ID).subscribe({
      next: (res: any) => {
        this.relationManagementForm.patchValue({
          Relation_ID: relation?.ID,
          RelationName: res?.[0]?.RelationName,
          IsActive: res?.[0]?.IsActive,
        });
        this.openRelationManagementModal('edit');
      },
      error: () => {},
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  transformToUpperCase(event: Event, controlName: string): void {
    const input = event.target as HTMLInputElement;
    input.value = input.value.toUpperCase();
    this.relationManagementForm.get(controlName)?.setValue(input.value);
  }
  
}
