import { Injectable } from '@angular/core';
import {
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor,
  HttpErrorResponse,
} from '@angular/common/http';
import { Observable, catchError, finalize, throwError } from 'rxjs';
import { AuthService } from './auth.service';
import { NgxSpinnerService } from 'ngx-spinner';

@Injectable()
export class TokenInterceptor implements HttpInterceptor {
  constructor(
    private authService: AuthService,
    private spinner: NgxSpinnerService
  ) {}

  intercept(
    request: HttpRequest<unknown>,
    next: HttpHandler
  ): Observable<HttpEvent<unknown>> {
    // Show the spinner only for outgoing HTTP requests
    this.spinner.show();

    return next.handle(request).pipe(
      catchError((error: HttpErrorResponse) => {
        if (error.status === 401) {
          // Handle unauthorized error (e.g., redirect to login page)
          this.authService.logOut();
        }
        if (
          error.message &&
          error.message.includes(
            'Unexpected value of the `Authorization` header provided'
          )
        ) {
          // Handle the case where Authorization header is null
          console.log('Authorization header is null');
          this.authService.logOut();
          // You can add your handling logic here
        }
        // Propagate the error so that the subscriber can handle it
        return throwError(error);
      }),
      finalize(() => {
        // Hide the spinner once the request is completed or fails
        this.spinner.hide();
      })
    );
  }
}
