import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';
 
@Injectable({
  providedIn: 'root',
})
export class ViewComplianceListService {
  url = environment.apiUrl;
 
  constructor(private http: HttpClient) {}
 
  getFillGrid(transactionID: any = '') {
    // Construct the base URL with the UserID parameter
    const baseUrl = `${environment.apiUrl}/api/compliance-management/view?UserID=${getCacheData('G_UserID')}`;
   
    // Add the TransactionID to the URL only if it is provided
    const url = transactionID ? `${baseUrl}&TransactionID=${transactionID}` : baseUrl;
 
    // Make the HTTP GET request with the appropriate URL
    return this.http.get(url, {
      headers: {
        Authorization: getCacheData('admin_token'),
        Accept: 'application/json',
        username: getCacheData('username'),
      },
    });
  }
 
 
  // getDataByTransactionNo(transactionNumber: any = 0) {
  //   return this.http.get(
  //     ${environment.apiUrl}/api/compliance-management/viewby-transactionnumber?UserID=${getCacheData('G_UserID')}&TransactionNumber=${transactionNumber},
  //     {
  //       headers: {
  //         Authorization: getCacheData('admin_token'),
  //         Accept: 'application/json',
  //         username: getCacheData('username'),
  //       },
  //     }
  //   );
  // }
  getDataByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/viewby-transactionnumber?UserID=${getCacheData('G_UserID')}&TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getViewComplianceByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/view-compliance?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getTransactionLogByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/transaction-log?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getViewByTransactionId(userId: any = 1001005, transactionId: any = '') {
    return this.http.get(
     `${environment.apiUrl}/api/compliance-management/viewby-transactionid?UserID=${getCacheData('G_UserID')}&TransactionNumber=${transactionId} `,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  getTransactionPrefix(userId: any = 1001005, transactionNumber: any = '') {
    return this.http.get(
        `${environment.apiUrl}/api/compliance-management/transaction-prefix?UserID=${getCacheData('G_UserID')}&TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getRequiredDetail(transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/required-detail?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getBindData(transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/api/master/compliance-management/bind-data?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getBindWithoutSbiTransactionNumber(transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/bind-without-sbi-transactionnumber?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getComplianceShowDetail(transactionNumber: any = '', searchType: any = '') {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/compliance-show-detail?TransactionNumber=${transactionNumber}&SearchType=${searchType}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
 
 
  updateCustomerBeneficiary(
    transactionNumber: any = '',
    isApprove: any = '',
    sType: any = '',
    remark: any = '',
    userId: any = '',
    customerBeneficiaryNo: any = ''
  ) {
    const body = {
      TransactionNumber: transactionNumber,
      IsApprove: isApprove,
      sType: sType,
      Remark: remark,
      UserID: getCacheData('G_UserID'),
      CustomerBeneficiaryNo: customerBeneficiaryNo,
    };
 
    return this.http.patch(
      `${environment.apiUrl}/api/compliance-management/updatecustben`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  approveCompliance(
    transactionNumber: any = '',
    remark: any = '',
   
  ) {
    const body = {  };
 
    return this.http.patch(
      `${environment.apiUrl}/api/compliance-management/approve?TransactionNumber=${transactionNumber}&Remark=${remark}&UserID=${getCacheData('G_UserID')}`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  rejectCompliance(
    transactionNumber: any = '',
    isMGTxn: any = '',
    isMTOSuccess: any = '',
    rejectReason: any = '',
   
    orgMsgID: any = ''
  ) {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/reject?TransactionNumber=${transactionNumber}&IsMGTxn=${isMGTxn}&IsMTOSuccess=${isMTOSuccess}&RejectReason=${rejectReason}&UserID=${getCacheData('G_UserID')}&OrgMsgID=${orgMsgID}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  viewByTransactionId(transactionNumber: any, showOFAC: any) {
    return this.http.get(
      `${environment.apiUrl}/api/compliance/view-compliance-hold-list/view`,
      {
        params: {
          TransactionNumber: transactionNumber,
          showOFAC: showOFAC,
        },
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getComplianceShowDetailWithSearchType(transactionNumber: string, searchType: string) {
    return this.http.get(
      `${environment.apiUrl}/api/compliance-management/compliance-show-detail`,
      {
        params: {
          TransactionNumber: transactionNumber,
          SearchType: searchType,
        },
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  getComplianceDetail(transactionNumber: any) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/view-compliance`,
      {
        params: {
          TransactionNumber: transactionNumber,
        },
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
 
  updateComplianceOnHold(
    transactionNumber: any = '',
    remark: any = '',
    // userId: any = 1001005
  ) {
    const body = {};
 
    return this.http.patch(
      `${environment.apiUrl}/api/compliance-management/onhold?TransactionNumber=${transactionNumber}&Remark=${remark}&UserID=${getCacheData('G_UserID')}`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  customerApprove(
    transactionNumber: any = '',
    remark: any = '',
    CustomerBeneficiaryNo:any =''
  ) {
    const body = {};
 
    return this.http.patch(
      `${environment.apiUrl}/api/compliance-management/customer-approve?TransactionNumber=${transactionNumber}&Remark=${remark}&UserID=${getCacheData('G_UserID')}&CustomerBeneficiaryNo=${CustomerBeneficiaryNo}`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  customerReject(payload: any) {
   
    return this.http.post(
      `${environment.apiUrl}/api/compliance-management/customer-reject
      `,
      payload,
      {
        headers:  {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  beneficiaryApprove(payload: any) {
   
    return this.http.post(
      `${environment.apiUrl}/api/compliance-management/beneficiary-approve
      `,
      payload,
      {
        headers:  {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  beneficiaryReject(payload: any) {
   
    return this.http.post(
      `${environment.apiUrl}/api/compliance-management/beneficiary-reject
      `,
      payload,
      {
        headers:  {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  transactionReject(payload: any) {
   
    return this.http.post(
      `${environment.apiUrl}/api/compliance-management/reject
      `,
      payload,
      {
        headers:  {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
 
 
  rejectionReason() {
    return this.http.get(
      `${environment.apiUrl}/api/compliance/compliance-rejection-reason/all-record`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}