import { Component, ElementRef, TemplateRef, ViewChild } from '@angular/core';
import { ViewCancelTransactionService } from '../services/view-cancel-transaction.service';
import * as XLSX from 'xlsx';
import { formatDate } from '@angular/common';
import { getCacheData } from '../utils/helper';
import { AuthService } from '../services/auth.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-view-cancel-transaction',
  templateUrl: './view-cancel-transaction.component.html',
  styleUrls: ['./view-cancel-transaction.component.css'],
})
export class ViewCancelTransactionComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;
  bsModalRef: BsModalRef;
  @ViewChild('viewLog') viewLogModal;

  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any = -1;
  agentBranchList: any = [];
  selectedAgentBranchList: any = 0;
  transactionNo: any = '';
  isLoading: boolean = false;
  receiptData: any = {};

  startDate: any;
  endDate: any;
  cancelledType: any = [
    {
      id: 0,
      type: 'All',
    },
    {
      id: 10,
      type: 'CancelWithOutCharges',
    },
    {
      id: 11,
      type: 'CancelWithCharges',
    },
    {
      id: 14,
      type: 'Remove Transaction',
    },
  ];
  selectedCancelledType: any = 0;

  statusList: any = [
    {
      id: -1,
      name: 'All',
    },
    {
      id: 0,
      name: 'Pending',
    },
    {
      id: 1,
      name: 'Processed',
    },
  ];
  selectedStatusList: any = -1;

  allCancelTransactions: any = [];
  selectedFileFormat: any = '1';

  tableCancelledType: any = [
    {
      id: 10,
      type: 'Cancellation / Refund charges',
    },
    {
      id: 11,
      type: 'Cancellation / do not refund charges',
    },
    {
      id: 0,
      type: 'Delete Cancellation Request',
    },
    {
      id: 29,
      type: 'MoveToInProcess',
    },
  ];

  selectedTableCancelledType: any = 10;
  LogData: any;

  constructor(
    private viewCancelTransactionSvc: ViewCancelTransactionService,
    private authSvc: AuthService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentAccountList(-1);
    this.getAgentBranchList(0);
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  getDefaultSelectedValue(status: any): any {
    if (status && status.TrStatusID === 7) {
      // Find the first item in tableCancelledType
      const defaultType = this.tableCancelledType[0];
      return defaultType ? defaultType.id : null; // Return id of the first item
    }
    return null; // Return null if status is not 7
  }

  // Handle change event of ng-select
  onChangeSelectedValue(newValue: any, index: number): void {
    // Update your data model with the selected value if needed
    this.allCancelTransactions[index].selectedTableCancelledType = newValue;
  }

  getCountryList() {
    this.viewCancelTransactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
      },
      error: () => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentAccountList(countryId);
    }
  }

  getAgentAccountList(countryId: any = 0) {
    this.viewCancelTransactionSvc.getAgentAccountList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: () => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId: any) {
    if (agentId != -1) {
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId: any = 0) {
    this.viewCancelTransactionSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
      },
      error: () => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }
    this.isLoading = true;

    let payload: any = {};
    payload = {
      TransactionNo: this.transactionNo,
    };
    this.viewCancelTransactionSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allCancelTransactions = res;
      },
      error: (err) => {
        this.allCancelTransactions = [];
        alert(err?.error?.message || 'Something went wrong');
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  displayByDate() {
    if (!this.startDate || !this.endDate) {
      alert('Please Select From & To Date!');
      return;
    }
    this.transactionNo = '';
    this.isLoading = true;

    let countryId = 0;
    if (this.selectedCountryList != 0) {
      countryId = this.selectedCountryList;
    }
    let payload: any = {};
    payload = {
      TransactionNo: '',
      DdlCountryID: countryId,
      DdlAgentsID: this.selectedAgentList,
      DrpAgentBranch: this.selectedAgentBranchList,
      StartDate: this.startDate,
      EndDate: this.endDate,
      DrpCancelledTypeID: this.selectedCancelledType,
      DrpStatusID: this.selectedStatusList,
      SubUserGroup_Id: getCacheData('SubUserGroup_Id'),
    };
    this.viewCancelTransactionSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allCancelTransactions = res;
      },
      error: (err) => {
        this.allCancelTransactions = [];
        alert(err?.error?.message || 'Something went wrong');
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_Cancel_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  get totalUSDAmount() {
    let totalAmount = 0;
    if (this.allCancelTransactions?.length) {
      for (let i = 0; i < this.allCancelTransactions.length; i++) {
        const element = this.allCancelTransactions[i];
        totalAmount += element.USDAmount;
      }
    }
    return totalAmount;
  }

  cancelTransaction(item) {
    console.log('item: ', item);
    const CancelReqDate =
      item?.CancelReqDate instanceof Date ? item.CancelReqDate : new Date();

    // Format the time using Intl.DateTimeFormat
    const time = new Intl.DateTimeFormat('en-US', {
      hour: 'numeric',
      minute: 'numeric',
    }).format(CancelReqDate);

    const payload = {
      TransactionNo: item?.TransactionNumber,
      App_ID: item?.App_ID,
      TrStatusID: item?.selectedTableCancelledType,
      txtRemarks: item?.Remarks,
      UserID: Number(getCacheData('G_UserID')),
      Date: item?.CancelReqDate,
      Time: time,
    };
    this.viewCancelTransactionSvc.CancelTransaction(payload).subscribe({
      next: (res: any) => {
        if (res) {
          if (this.transactionNo) {
            this.filterByTransactioNo();
          } else {
            this.displayByDate();
          }
          alert('Record updated successfully.');
        }
      },
      error: (err) => {
        //alert(err?.erro?.message);
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  getlog(TransactionNumber: any) {
    this.ViewTransactionStatusForAgentSvc.getLog(TransactionNumber).subscribe(
      (res) => {
        this.openModal(this.viewLogModal);
        this.LogData = res;
      }
    );
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.TransactionNumber;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }
}
